/***************************************************************************
 * JMMC project ( http://www.jmmc.fr ) - Copyright (C) CNRS.
 ***************************************************************************/

/***************************************************************************/

// To see how the web tool is defined, look at amdlibPreferences.i

/*************************************************************/
/** Includes *************************************************/
/*************************************************************/

require, "bessel.i";

/*************************************************************/

func amdlibCalibrate(void)
    /* DOCUMENT amdlibCalibrate
       
       DESCRIPTION
       AMBER OI data calibration routines.
       Two sets of calibration routines are implemented here. The user has the
       choice of the routines he wishes to use.
       - Millour et al. routines
       (Millour, Valat, Petrov & Vannier 2008, SPIE, 7013, 132)
       - Lebouquin routines (ref?)
   
       VERSION
       $Revision: 14480 $

       FUNCTIONS
       Pre-calibration routines:
       - amdlibCheckAllWavelengthTable   : Correct for the offset in the
                                           OI_WAVELENGTH table by comparing the
                                           AMBER_SPECTRUM with a reference
                                           telluric spectra.
       - amdlibSearchAllStarDiameters    : build a calibrator database from
                                           the local catalogs.
       
   Lebouquin routines:
   - amdlibCalibrateAllOiData        : Compute calibrated data (see help for
                                       details)
   
   Millour routines:
   - amdlibComputeAllTransferFunction: compute transfer function
   - amdlibShowTransferFunctionVsTime: Plot transfer function as a function
                                       of time
   - amdlibShowTransferFunctionVsWlen: Plot transfer function as a function
                                       of wavelength
   - amdlibCalibrateOiData           : calibrate OI data files

       SEE ALSO
       yocoFile, amdlib
    */
{
    version = strpart(strtok("$Revision: 14480 $",":")(2),2:-2);
    if (am_subroutine())
    {
        help, amdlibCalibrate;
    }   
    return version;
}

/****************************************************************/
/*** Data structures ********************************************/
/****************************************************************/

struct oidata
/* DOCUMENT oidata

   DESCRIPTION
   Simplified OI DATA structure, to store AMBER datas

   PARAMETERS
   string name;
   string file;
   double time;
    
   pointer vis2;
   pointer vis2Err;

   pointer phi;
   pointer phiErr;

   pointer clos;
   pointer closErr;

   pointer wlen;
   pointer band;

   pointer uv;
   pointer uv1;
   pointer uv2;
       
   SEE ALSO:
*/
{
    string name;
    string file;
    double time;
    
    pointer vis2;
    pointer vis2Err;

    pointer dPhi;
    pointer dPhiErr;

    pointer dVis;
    pointer dVisErr;

    pointer clos;
    pointer closErr;

    pointer wlen;
    pointer band;

    pointer uv;
    pointer uv1;
    pointer uv2;
};

/*************************************************************/

// Interferometry diameters catalogs in VIZIER
// Description, VIZIER name, diameter column name, error on the diameter column
amdlibDIAMETERS_CATALOGS = [["CHARM (Richichi, 2002, A&A, 386, 492)",
                             "J/A+A/386/492", "cDiam", "e_cDiam"],
                            ["Borde et al. (Borde+, 2002, A&A, 393, 183)",
                             "J/A+A/393/183", "LDD", "e_LDD"],
                            ["CHARM2 (Richichi+, 2005, A&A, 431, 773)",
                             "J/A+A/431/773", "UD", "e_UD"],
                            ["Merand et al. (Merand+, 2005, A&A, 433, 1155)",
                             "J/A+A/433/1155", "UDdiamKs", "e_UDdiam"],
                            ["ESO calibrators programme (Richichi+, 2005, A&A, 434, 1201)",
                             "J/A+A/434/1201","Diam", "e_Diam"],
                            ["CADARS (Pasinetti-Fracassini+ 2001, A&A, 367, 521)",
                             "II/224/cadars","Diam", ""],
                            ["Van Belle et al. (Van Belle et al., 2008, ApJSS, 176, 276)",
                             "J/ApJS/176/276/stars", "theta", "e_theta"]
                            ];

/****************************************************************/
/*             Local version of the catalogs                  */
/****************************************************************/

func _amdlibLoadCatalog(inputCatalogFile, &dcat, &ecat, &racat, &decat, &idcat, dname, ename, id)
    /* DOCUMENT _amdlibLoadCatalog(inputCatalogFile, &dcat, &ecat, &racat, &decat, &idcat, dname, ename, id)

       DESCRIPTION
       Load the binary table contains in FITS file name+".fits[2]" and grows the arrays.

       PARAMETERS
       - inputCatalogFile: filename of the catalog (J_AA434_1201).
       - dcat            : diameter (mas)
       - ecat            : diameter error (mas)
       - racat           : RA (deg)
       - decat           : DEC (deg)
       - idcat           : the same as 'id'
       - dname           : name of the column containing the diameter (in mas)
       - ename           : name of the column containing the diameter error
       (in mas)
       - id              : integer that will reference the catalog.

       If the diameter error is not specified, or egal to 0, then the
       error will be arbitrarely set to 30% of the diameter.

       EXAMPLES

       SEE ALSO
    */
{
    yocoLogTrace,"_amdlibLoadCatalog()";
    local fh, name; 
    
    /* Check the file */
    message = "Choose the catalog";
    if ( !amdlibFileChooser(message, inputCatalogFile, 3) ||
         !open(inputCatalogFile,"r",1) )
    {
        yocoError,"'inputCatalogFile' is not valid:",inputCatalogFile;
        return 0;
    }

    /* Open the file and go to the first binary table HDU */
    yocoLogInfo,"Load local catalog:",inputCatalogFile;
    fh = cfitsio_open(inputCatalogFile);
    cfitsio_goto_hdu,fh,2;

    /* Check the units */
    unit = cfitsio_get(fh,"TUNIT"+pr1(cfitsio_get_colnum(fh,,dname)));
    if (unit=="mas") {
        fact = 1.0;
    } else if (unit=="arcsec"){
        fact = 1e3;
    } else {
        yocoError,"Cannot recognise the unit (correct unit in catalog file).";
        return 0.0;
    }

    /* Load coordinates */
    grow, racat, yocoStrTime(  cfitsio_read_column(fh,"RAJ2000") ) / 12. * 180;
    grow, decat, yocoStrAngle( cfitsio_read_column(fh,"DEJ2000") );
    grow, dcat, cfitsio_read_column(fh,dname) * fact;

    /* Eventually load the error. If not given or 0, the error
       will be set to 30% */
    tmp = cfitsio_read_column(fh,dname) * fact * 0.3;
    err = ( (is_array(ename) && ename!="") ? cfitsio_read_column(fh,ename) * fact : tmp*0.0 );
    grow, ecat, err + tmp*(err==0);

    /* Eventually load the id */
    if ( is_array(id) ) grow, idcat, int(tmp*0.0 + id);

    /* close file */
    cfitsio_close,fh;
  
    yocoLogTrace,"_amdlibLoadCatalog done.";
    return 1;
}

/*************************************************************/

func amdlibSearchAllStarDiameters(inputDir=, calibTextFile=, queryAllStars=, catalogsDir=)
    /* DOCUMENT amdlibSearchAllStarDiameters(inputDir=, calibTextFile=, queryAllStars=, catalogsDir=)
   
       DESCRIPTION
       Search the diameter of the calibration star contained in the directory
       inputDir. The diameters are searched in the local catalogs, and the
       search is performed on the coordinates stored in logfile.

       The diameters are extracted in catalogs (sorted by confidence order):
       J_AA_433_1155, J_AA_393_183, II_300_jsdc, J_AA_434_1201,
       J_AA_431_773, II_224_cadars.
   
       The catalogs should be placed in the directoty given by the input
       'catalogsDir'. The default location can be changed in the preferences.
   
       The diameters are stored in the ASCII file 'calibTextFile'.
       If it already exists, the file is only updated. The default filename
       for 'calibTextFile' can be changed in the preferences.

       Once created by this function, the 'calibTextFile' can be updated
       manually to add/update the diameters of calibration stars.
   
   PARAMETERS
   - inputDir      : directory containing the DATA (RAW_DATA or OIDATA)
   - calibTextFile : name of the output ASCII file.
   - catalogsDir   : location of the catalogs.
   - queryAllStars : if set to 1, search the diameters of all stars and
                     not only calibrators. Defaults is 0.
   
       EXAMPLES
       > amdlibSearchAllStarDiameters, inputDir="./";
   
       SEE ALSO
    */
{
    yocoLogInfo,"amdlibSearchAllStarDiameters()";
    local racat, decat, diamcat, errcat, data, titles;
    local diam, diamerr, ra, dec, message, tab, tit;
  
    /* Check simple parameters */
    if ( !amdlibCheckCalibTextFile(calibTextFile) ||
         !amdlibCheckCatalogsDir(catalogsDir) )
    {
        yocoError, "Check function's parameters";
        return 0;
    }

    /* Select directory containing files to be treated */
    message = "Choose the input directory";
    if ( !amdlibFileChooser(message, inputDir, 3)  ||
         !amdlibCheckDir(inputDir, 0) )
    {
        yocoError,"'inputDir' not specified correctly.";
        return 0;
    }

    /* Read the log of the inputDir */
    if ( !amdlibReadLogFromDir(inputDir, tab, tit) )
    {
        yocoError, "Could not read logfile of 'inputDir'";
        return 0;
    }

    /* Check if data are present */
    if ( !is_array(tab) ){
        yocoLogError,"No OIDATA on this directory.";
        return 0;
    }

    /* Keep only the observation with OBS.CATG=CALIB and OBS.TYPE=OBJECT,
       execpt if queryAllStars==1 */
    if ( !queryAllStars )
    {
        id = where(  amdlibLogGet(tab, tit, "obs_catg") == "CALIB" &
                     amdlibLogGet(tab, tit, "obs_type") == "OBJECT" );
        tab = tab(,id);
    } else
    {
        yocoLogInfo, "Search for diameter of all stars, not only calibration stars.";
    }

    /* Check if data are present */
    if ( !is_array(tab) )
    {
        yocoLogError,"No calibration star in this directory.";
        return 0;
    }
  
    /* Get a list of name with RA/DEC */
    name   = amdlibLogGet(tab, tit, "object_name");
    ra     = yocoStr2Double( amdlibLogGet(tab, tit, "object_ra")  );
    dec    = yocoStr2Double( amdlibLogGet(tab, tit, "object_dec") );
  
    /* Clean multiple entry based on target name */
    ra     = yocoListClean(ra,name);
    dec    = yocoListClean(dec,name);
    name   = yocoListClean(name);

    /* Prepare an array for the catalog id */
    catname  = ["J_AA_433_1155","J_AA_393_183","II_300_jsdc","J_AA_434_1201","J_AA_431_773","II_224_cadars","none"];
    catfiles = catalogsDir + "/" + catname + ".fits";
    catid   = indgen(numberof(catname)+1);

    /* Read the catalogs (very fast)
       FIXME: Miss the Van bell */
    diamcat = errcat = racat = decat = idcat = [];
    if (
        !oiFitsLoadCatalog(catfiles(1),diamcat,errcat,racat,decat,idcat,"UDdiamH","e_UDdiam",1) ||
        !oiFitsLoadCatalog(catfiles(2),diamcat,errcat,racat,decat,idcat,"UDDH","e_UDDH",2) ||
        !oiFitsLoadCatalog(catfiles(3),diamcat,errcat,racat,decat,idcat,"UDDH","e_LDD",3)  ||
        !oiFitsLoadCatalog(catfiles(4),diamcat,errcat,racat,decat,idcat,"Diam","e_Diam",4) ||
        !oiFitsLoadCatalog(catfiles(5),diamcat,errcat,racat,decat,idcat,"UD","e_UD",5)     ||
        !oiFitsLoadCatalog(catfiles(6),diamcat,errcat,racat,decat,idcat,"Diam",,6) )
    {
        yocoLogError,"Cannot load the local catalogs.";
        return 0;
    }   

    /* Prepare the outputs  */
    diam = diamerr = array(-1.0,dimsof(name));
    distance = catalog = array("--",dimsof(name));

    /* Loop on stars */
    for ( i=1 ; i<=numberof(name) ; i++)
    {
        /* Look for the stars by matching the coordinates (seconds) */
        dist =  abs(ra(i)-racat, dec(i)-decat)*60.*60;
        ids = id = where( dist < 5.0 & diamcat > 0.0 );

        /* If no diameter found */
        if ( numberof(id)<1 )
        {
            yocoLogWarning,name(i)+" has no diameter found.";
            continue;
        }

        /* Keep the diameter of the catalog with the best confidence,
           that is of minimum value of idcat */
        ids = ids( sort(idcat(ids)) );
        id = id( where(idcat(ids)==min(idcat(ids))) );
    
        /* Fill the diameter as the weighted average */
        tmp =  1./errcat(id)(*)^2.0;
        diam(i) = (diamcat(id)(*) * tmp )(sum) / tmp(sum);
    
        /* Fill the error as the average error */
        diamerr(i) = errcat(id)(*)(avg);

        /* Fill the list of catalogs */
        sep = (numberof(id)>1 ? ":" : "");
        catalog(i) = (yocoListClean(catname(idcat(id)))+sep)(*)(sum);

        /* Fill the distance */
        distance(i) = swrite(format="%.1f"+sep,dist(id)(*))(sum);
    
        /* Verbose information */
        main = swrite(format="%s has diameter of %.2f+-%.2fmas from %s\n All references are:",
                      name(i),diam(i),diamerr(i),catalog(i));
        info = swrite(format="%.2f+-%.2f (%.1f\") in %s\t",diamcat(ids),errcat(ids),
                      dist(ids),catname(idcat(ids)));
        yocoLogInfo,main,info;
    }
    /* End loop on stars */


    /* Now create/update the ASCII file that contains the
       diameters of the used calibrators.
       First load the already existing ASCII file, or
       just create the required columns */
    if ( open(calibTextFile,"r",1) )
    {
        amdlibReadAsciiFile, calibTextFile, data, titles;
        ncat = data(where(titles=="Star_name")(1),);
    } else {
        titles = ["Star_name","OB_name","RA(deg)",
                  "DEC(deg)","diam(mas)","diamErr(mas)","dist(\")","Catalog"];
    }

    /* Loop on stars */
    for ( i=1 ; i<=numberof(name) ; i++)
    {
        /* If no diameter found */
        if ( anyof(ncat == name(i)) )
        {
            yocoLogInfo,name(i)+" is already in file:",calibTextFile;
            continue;
        }

        /* Else add the data at the end */
        grow, data, [array("--",numberof(titles))];
        data( where(titles=="Star_name")(1) ,0) = name(i);
        data( where(titles=="RA(deg)")(1) ,0)   = swrite(format="%.7f",ra(i));
        data( where(titles=="DEC(deg)")(1) ,0)  = swrite(format="%.7f",dec(i));
        data( where(titles=="diam(mas)")(1) ,0) = pr1(diam(i));
        data( where(titles=="diamErr(mas)")(1) ,0) = pr1(diamerr(i));
        data( where(titles=="dist(\")")(1) ,0)  = distance(i);
        data( where(titles=="Catalog")(1) ,0)   = catalog(i);
        
    }

    /* create, write and close the file */
    yocoLogInfo,"Diameters written in file",calibTextFile;
    amdlibWriteAsciiFile, calibTextFile, data, titles;

    /* Some final verbose information */
    if ( anyof(diam==-1) || anyof(diamerr==-1) )
    {
        yocoLogWarning,"Diameters are missing. Fill them manually in file:",calibTextFile;
    }
  
    yocoLogInfo,"amdlibSearchAllStarDiameters done";
    return 1;
}

/*************************************************************/
/** Calibration routines from Millour et al. 2008 ************/
/*************************************************************/

func amdlibComputeTransferFunction(inputOiFile=, calDiam=, calDiamErr=, outputOiFile=)
    /* DOCUMENT amdlibComputeTransferFunction(inputOiFile=, calDiam=, calDiamErr=, outputOiFile=)

       DESCRIPTION
       From an AMBER averaged data file, compute the "transfer function"
       file by dividing the visibility by the expected visibility given
       in calDiam. An optional calDiamErr error is taken into account
       and propagated to the final error bar.

       PARAMETERS
       - inputOiFile : 
       - calDiam     : 
       - calDiamErr  : 
       - outputOiFile: 

       RESULTS
       A file of "transfer function" type.

       EXAMPLES


       SEE ALSO:
    */
{
    yocoLogInfo, "amdlibComputeTransferFunction()";
    local tmp, caldir;
    
    /* If the function is called without argument, a file browser popups to
       choose the correct file */
    message = "Enter the calibration star OI file to load";
    if (amdlibFileChooser(message, inputOiFile, 3) == 0)
    {
        yocoError,"'inputOiFile' not specified correctly.";
        return 0;
    }

    /* If calDiam is void, then try to query CDS */
    if ( is_void(calDiam) || !yocoTypeIsNumericalScalar(calDiam))
    {
        yocoError,"calDiam should be a scalar number.";
        return 0;
    }

    /* Default for error */
    if ( is_void(calDiamErr) ) calDiamErr = 0.0;
    
    /* Read the calibration star OI_VIS file */
    _amdlibReadVis, oiWave, amberPhot, oiVis, oiVis2, oiVis3, amberOpd, amberInsCfg,
        oiArray, oiTarget, amberSpectrum, inputOiFile=inputOiFile;
    
    /* Check the number of frames */
    if ( oiVis2.nbFrames != 1 )
    {
        yocoError, "Selected file has more than 1 frame, could not calibrate it.",
            "Perform frame selection first.";
        return 0;
    }

    /* Extract wlen in meters */
    wlen = (*oiWave.wlen) * 1e-9;

    /* Calibrate the vis2, loop on the bases */
    for ( iBase=1 ; iBase <= oiVis2.nbBases ; iBase++ )
    {
        /* local u,v in meters */
        base    = abs( (*oiVis2.table)(iBase).uCoord, (*oiVis2.table)(iBase).vCoord );
        vis2    = *(*oiVis2.table)(iBase).vis2;
        vis2Err = *(*oiVis2.table)(iBase).vis2Error;
        
        /* calibrate */
        z        = base / wlen * mas2rad;
        vis2theo = abs( yocoMathAiry( z * calDiam ) )^2;
   
        /* Compute error */
        visMax   = abs( yocoMathAiry( z * (calDiam+calDiamErr) ) );
        visMin   = abs( yocoMathAiry( z * (calDiam-calDiamErr) ) );
        vis2theoErr  = abs(visMin - visMax)^2;

        /* Put data back */
        *(*oiVis2.table)(iBase).vis2       = vis2 / vis2theo;
        *(*oiVis2.table)(iBase).vis2Error  = sqrt( (vis2Err^2 * vis2theo^2 + vis2^2 * vis2theoErr^2) / (vis2theo^4) );
    }
    /* End loop on the bases */
    
    /* Set the default output file name and product directory */
    if (is_void(outputOiFile))
    {
        yocoFileSplitName,inputOiFile,tmp,calname,calext;

        /* Find the default for product dir */
        if ( !amdlibCheckProductDir(caldir, 1, suffix="TRA", up=1, inputDir=tmp) )
        {
            yocoError, "Could not construct the default directory for the OIDATA_TRA product";
            return 0;
        }

        /* build the name */
        outputOiFile = caldir + yocoStrReplace(calname,"AVG","TRA") + calext;
    }

    
    /* Handle compressed file */
    compress = amdlibCheckCompress(outputOiFile);
    
    /* Save OI-FITS resulting file. */
    yocoLogInfo, "Writing file", outputOiFile;
    if ( amdlibWriteOiFile( outputOiFile,
                            pointer(nil), &oiArray, 
                            &oiTarget, &oiWave, &amberPhot,
                            &oiVis, &oiVis2, &oiVis3,
                            &amberOpd, &amberSpectrum, errMsg) == amdlibFAILURE )
    {
        yocoError, "Could not write OI files", errMsg;
        return 0;
    }

    /* Copy all keywords that could be useful for use afterwards */
    _amdlibDumpAllKeywords, 1, inputOiFile, outputOiFile;

    /* Update keyword related to calibration */
    outFh = cfitsio_open(outputOiFile, "a", overwrite=1);
    cfitsio_goto_hdu,outFh,1;
    cfitsio_set, outFh, "HIERARCH ESO PRO CATG", "TRANSFER_FUNCTION",
        "By amdlibComputeTransferFunction";
    cfitsio_close_file, outFh;
    
    /* Handle compressed file */
    if ( !amdlibCompress(outputOiFile,compress) )
    {
        yocoError, "Could not compress the output OIDATA_TRA file";
        return 0;
    }
    
    yocoLogInfo, "amdlibComputeTransferFunction done";
    return 1;
}

/***************************************************************************/

func amdlibComputeAllTransferFunction(inputDir=, outputDir=, calibTextFile=, logFile=, overwrite=)
    /* DOCUMENT amdlibComputeAllTransferFunction(inputDir=, outputDir=, calibTextFile=, logFile=, overwrite=)

       DESCRIPTION
       Night transfer function computation on a bunch of files contained in a
       directory.

       PARAMETERS
       - inputDir     : Data directory name
       - outputDir    : 
       - calibTextFile: 
       - logFile      : 
       - overwrite    : 

       RESULTS

       EXAMPLES

       SEE ALSO:
    */
{
    yocoLogInfo, "amdlibComputeAllTransferFunction()";

    /* Check simples parameters */
    if ( !amdlibCheckOverwrite(overwrite)       ||
         !amdlibCheckGraphicsDpi(graphicsDpi)   ||
         !amdlibCheckCalibTextFile(calibTextFile) )
    {
        yocoError, "Check function's parameters";
        return 0;
    }

    /* Select directory containing files to be treated */
    message = "Choose the input directory (OIDATA_AVG)";
    if ( !amdlibFileChooser(message, inputDir, 3)  ||
         !amdlibCheckDir(inputDir, 0) )
    {
        yocoError,"'inputDir' not specified correctly.";
        return 0;
    }
    
    /* Verbose output */
    yocoLogInfo, "Computing all Transfer Function for directory"+
        " (overwrite="+pr1(overwrite)+") :", inputDir;
    
    /* Check outputDir */
    if ( !amdlibCheckProductDir(outputDir, 1, overwrite, suffix="TRA", up=1, inputDir=inputDir) )
    {
        yocoError, "Could not construct 'outputDir'";
        return 0;
    }

    /* Verbose output */
    yocoLogInfo, "Product directory is:",outputDir;

    /* If existing but overwrite is void, ask the user */
    if (is_void(overwrite))
    {
        overwrite = yocoGuiInfoBox("Directory " +
                                   yocoStrrtok(outputDir,"/")(2) + 
                                   " already exists. Re-average all OIDATAs ?\n",
                                   butText=["YES","NO!"],butReturn=[1,0],
                                   dpi=graphicsDpi);
    }
    
    /* Read the calibTextFile */
    if ( !amdlibReadAsciiFile( calibTextFile, data, titles) )
    {
        yocoError,"Cannot load the calibration data file.","Please (re)compute it.";
        return 0;
    }
    
    /* Extract the info */
    calNames    = data(where(titles=="Star_name"),);
    calDiamz    = yocoStr2Double( data(where(titles=="diam(mas)"), ));
    calDiamzErr = yocoStr2Double( data(where(titles=="diamErr(mas)"), ));


    /* Read the log of the inputDir */
    if ( !amdlibReadLogFromDir(inputDir, logTable, titles) )
    {
        yocoError, "Could not read logfile of 'inputDir'";
        return 0;
    }

    /* Extract the info from the Log */
    tabFiles = logTable(where(titles=="fileName"), );
    tabNames = logTable(where(titles=="object_name"), );
    proTypes = logTable(where(titles=="pro_catg"), );

    /* Get the calibration files only */
    calibObs   = where(proTypes=="CALIB_AVERAGED");
    calibFiles = tabFiles(calibObs);
    calibNames = tabNames(calibObs);
    nCalFiles  = numberof(calibFiles);

    /* Check if nothing has to be done */
    if ( nCalFiles < 1 )
    {
        yocoLogWarning, "No OIDATA_AVG files to be processed into OIDATA_TRA... exit!";
        return 1;
    }
    
    /* Loop on files */
    for(kFile=1;kFile<=nCalFiles;kFile++)
    {
        inputOiFile=inputDir+calibFiles(kFile);

        /* Verbose output */
        yocoLogInfo, "amdlibComputeAllTransferFunction is now working in file:",
            inputOiFile;
                
        /* Get expected calibrator visibility */
        test = where(strmatch(calNames,calibNames(kFile)));

        /* Case no diameter */
        if( numberof(test) < 1 )
        {
            yocoLogWarning, "The star star \'"+calibNames(kFile)+"' has no diameter !",
                "Not including it in the transfer function";
            continue;
        }

        /* Get the diameter */
        calDiam = avg(calDiamz(*)(test));
        calDiamErr = avg(calDiamzErr(*)(test));
        
        /* Case diameter is 0 */
        if( calDiam == 0 )
        {
            yocoLogWarning, "The star star \'"+calibNames(kFile)+"' has a diameter of 0 !",
                "Not including it in the transfer function";
            continue;
        }

        /* Verbose output */
        yocoLogInfo, "Calibration star '"+calibNames(kFile)+"' has diameter of "+
            swrite(format="%.3f +- %.3f mas", calDiam, calDiamErr);
        
        /* Construct output OI file name.
           Use the same rule as amdlibComputeTransferFunction */
        yocoFileSplitName,inputOiFile,caldir,calname,calext;
        outputOiFile = outputDir + yocoStrReplace(calname,"AVG","TRA")+calext;
        
        /* Check if produced file already exist if yes,
           skip this file expect if overwrite==1 */
        if ( yocoTypeIsFile(outputOiFile) && (overwrite==0) )
        {
            yocoLogInfo,"Skipped because the following file already exist:",
                outputOiFile;
            continue;
        }
        
        /* Compute the tranfer function */
        if( amdlibComputeTransferFunction(inputOiFile=inputOiFile,
                                          calDiam=calDiam,
                                          calDiamErr=calDiamErr,
                                          outputOiFile=outputOiFile) != 1)
        {
            yocoError, "Could not compute OIDATA_TRA for this file.";
            continue;
        }

    }
    /* End loop on files */

    /* Verbose output */
    yocoLogInfo, "amdlibComputeAllTransferFunction done";
    return 1;
}

/***************************************************************************/

func amdlibLoadAllOiData(filesType, &dataStruct, inputDir=, verbose=, crl=, crr=, dVisFromV2=)
    /* DOCUMENT amdlibLoadAllOiData(filesType, &dataStruct, inputDir=, verbose=, crl=, crr=, dVisFromV2=)

       DESCRIPTION
       Load only the files marked as the keyword given in filesType and
       fill in an oidata structure

       PARAMETERS
       - filesType : must be "SCIENCE_AVERAGED", "CALIB_AVERAGED",
                     "TRANSFER_FUNCTION" or "SCIENCE_CALIBRATED"
       - dataStruct: an OI_DATA structure containing the data of the full night
       - inputDir  : Data directory name, asked if nothing is entered
       - verbose   : 
       - crl       : optionnal wavelength-cropping for the files to remove
                     spurious data at the inter-bands (H-K or J-H), left of
                     the dataset.
       - crr       : optionnal wavelength-cropping for the files to remove
       - dVisFromV2: 

       RESULTS
       An OI_DATA structure

       EXAMPLES


       SEE ALSO:
    */
{
    if(is_void(dVisFromV2))
        dVisFromV2=1;
    
    if(open(inputDir,"r",1))
    {
        tmpDir = cd(".");
        cd, inputDir;
    }
    else
        return 0;
       
    VIS2 = VIS2_ERR =
        PHI = PHI_ERR =
        CLOS = CLOS_ERR =
        WLEN = TIME = NAMES =
        ORIG_V2 = ORIG_V2_ERR =
        FILENAMES = BANDWIDTH = UV = UV1 = UV2 = dataStruct = [];
    
    /* Select directory containing files to be treated */
    message = "Choose the product directory (OI_DATA)";    
    if (amdlibFileChooser(message, inputDir,3) == 0)
    {
        yocoError,"'inputDir' not specified correctly.";
        return 0;
    }
    
    if(is_void(verbose))
        verbose=1;
    
    amdlibGetLogName, inputDir, logFile;
    amdlibReadLog, logFile, logTable, titles;

    tabFiles = logTable(where(titles=="fileName"), );
    tabDates = logTable(where(titles=="date_obs"), );
    tabNr    = logTable(where(titles=="calib_nr"), );
    proTypes = logTable(where(titles=="pro_catg"), );
    tabTypes = logTable(where(titles=="obs_catg"), );
    OBJTypes = logTable(where(titles=="obs_type"), );
    tabIds   = logTable(where(titles=="P2VM_ID"), );
    tabTplNr = logTable(where(titles=="Template_nr"), );
    tabNames = logTable(where(titles=="object_name"), );
    
    sciObs = where((proTypes==filesType));

    sciFiles = tabFiles(sciObs);
    sciNames = tabNames(sciObs);
    nSciFiles = numberof(sciFiles);

    if(nSciFiles==0)
        return 0;
    
    for(kFile=1;kFile<=nSciFiles;kFile++)
    {
        inputOiFile=inputDir+"/"+sciFiles(kFile);
        
        write,"Loading",sciFiles(kFile);
        
        if(amdlibLoadOiData( wlen, bandWidth, time, fringeSNR, sqVis, sqVisErr,
                             uvCoord, bandFlag,
                             pistonOPDArray, sigmaPistonArray,
                             diffVisAmp, diffVisAmpErr,
                             diffVisPhase, diffVisPhaseErr,
                             vis3Amp, vis3AmpErr,
                             vis3Phase, vis3PhaseErr, cpxVis, cpxVisErr,
                             visCovRI, uv1Coord, uv2Coord,
                             fluxSumPiPj, fluxRatPiPj,
                             fluxProdPiPj, spectrum, spectrumErr,
                             inputOiFile=inputOiFile) !=0)
        {
            nbFrames = dimsof(sqVis)(4);
            nbWlen = dimsof(sqVis)(3);
            nbBases = dimsof(sqVis)(2);

            U = uvCoord.u;
            V = uvCoord.v;
            if(!is_void(vis3Phase))
            {
                U1 = transpose(array(uv1Coord.u,1));
                V1 = transpose(array(uv1Coord.v,1));
                U2 = transpose(array(uv2Coord.u,1));
                V2 = transpose(array(uv2Coord.v,1));
                uv1 = [U1,V1];
                uv2 = [U2,V2];
            }
            uv = [U,V];
            
            grow, VIS2,        &sqVis(,crl:crr,);
            grow, VIS2_ERR,    &sqVisErr(,crl:crr,);
            if(!is_void(vis3Phase))
            {
                grow, CLOS,      &vis3Phase(,crl:crr,);
                grow, CLOS_ERR,  &vis3PhaseErr(,crl:crr,);
                grow, UV1,       &uv1;
                grow, UV2,       &uv2;
            }
            else
            {
                dummy=[];
                grow, CLOS,      &dummy;
                grow, CLOS_ERR,  &dummy;
                grow, UV1,       &dummy;
                grow, UV2,       &dummy;
            }
                
            grow, PHI,       &diffVisPhase(,crl:crr,);
            grow, PHI_ERR,   &diffVisPhaseErr(,crl:crr,);
            grow, WLEN,      &wlen(crl:crr);
            grow, TIME,      avg(time);
            grow, NAMES,     sciNames(kFile);
            grow, BANDWIDTH, &bandWidth(crl:crr);
            grow, FILENAMES, inputOiFile;
            grow, UV,        &uv;
        }
    }

    dataStruct = array(oidata(),nSciFiles);
    dataStruct.wlen = WLEN;
    dataStruct.band = BANDWIDTH;
    dataStruct.vis2 = VIS2;
    dataStruct.vis2Err = VIS2_ERR;
    if(!is_void(CLOS))
    {
        dataStruct.clos = CLOS;
        dataStruct.closErr = CLOS_ERR;
    }
    dataStruct.dPhi = PHI;
    dataStruct.dPhiErr = PHI_ERR;
    dataStruct.name = NAMES;
    dataStruct.file = FILENAMES;
    dataStruct.time = TIME;
    dataStruct.uv = UV;
    if(!is_void(vis3Phase))
    {
        dataStruct.uv1 = UV1;
        dataStruct.uv2 = UV2;
    }

    if(dVisFromV2==1)
        _amdlibComputeDiffVisFromV2,dataStruct;

    cd, tmpDir
}

/***************************************************************************/

func amdlibShowTransferFunctionVsWlen(inputDir=, verbose=, dVisFromV2=)
    /* DOCUMENT amdlibShowTransferFunctionVsWlen(inputDir=, verbose=, dVisFromV2=)

       DESCRIPTION
       Plot a series of "TRANSFER_FUNCTION" files with the "TRA" suffix as a function of wavelength

       PARAMETERS
       - inputDir  : 
       - verbose   : 
       - dVisFromV2: 

       RESULTS


       EXAMPLES
       > amdlibShowTransferFunctionVsWlen 

       SEE ALSO:
    */
{
    local transDir;
    colors= ["red","green","blue","magenta","cyan","yellow","black"];
  
    /* Select directory containing files to be treated */
    message = "Choose the product directory (OI_DATA)";    
    if (amdlibFileChooser(message, inputDir, 3) == 0)
    {
        yocoError,"'inputDir' not specified correctly.";
        return 0;
    }
    
    yocoGuiWinKill;
    amdlibGetPreference,"graphicsDpi",dpi;
    yocoNmCreate,1,1,3,width=638*dpi/75,height=825*dpi/75, V = [0.12,0.73,0.1,0.9],dy=0;
    
    amdlibLoadAllOiData , "TRANSFER_FUNCTION", trans, inputDir=inputDir, dVisFromV2=dVisFromV2;
    if(is_void(trans))
    {
        /* check for the transfer function directory */
        if ( !amdlibCheckProductDir(transDir, 0, suffix="TRA", up=1, inputDir=inputDir+"/") )
        {
            yocoError, "Could not find the directory with the OIDATA_TRA files", , 1;
            return 0;
        }
        
        amdlibLoadAllOiData , "TRANSFER_FUNCTION", trans, inputDir=transDir, dVisFromV2=dVisFromV2;
        
        if(is_void(trans))
            yocoError,"No transfer function !";
    }
        
    // Decide max wavelength
    maxL = 0;
    for(iWin=1;iWin<=3;iWin++)
    {
        N = numberof(trans.time);
        for(k=1;k<=N;k++)
        {
            v2 = (*trans.vis2(k))(,,1);
            v2Err = (*trans.vis2Err(k))(,,1);

            if(anyof(v2+v2Err>maxL))
                maxL = 1.1*max(v2+v2Err);
        }
    }
    maxL = min(maxL, 1.1);
    
    for(iWin=1;iWin<=3;iWin++)
    {
        plsys,iWin;
        N = numberof(trans.time);
        for(k=1;k<=N;k++)
        {
            wlen = *trans.wlen(k);
            v2 = (*trans.vis2(k))(,,1);
            v2Err = (*trans.vis2Err(k))(,,1);
            
            nbBases = dimsof(v2)(2);
            
            if(iWin<=nbBases)
            {
                yocoPlotWithErrBars,
                    v2(iWin,,1),v2Err(iWin,,1),wlen,color=colors(k%7);
            }
        }
        limits,,,0.0,maxL;
    }
    yocoNmXytitles,"Wavelength","V^2";
    plsys,1;
    d = yocoStrReplace(yocoStrrtok(inputDir, "/")(2));
    pltitle,"Transfer function for "+d;
    hcps,"./TRANSFER_FUNCTION_WLEN.ps";
}

amdlibShowTransFuncWlen = amdlibShowTransferFunctionVsWlen;

/***************************************************************************/

func _amdlibGetColor(intensite, phase)
/* DOCUMENT _amdlibGetColor(intensite, phase)
   Produces a [r,g,b] array starting from an intensity and phase, where phase=0
   and phase=2pi is red, phase=2pi/3 is green and phase=4pi/3 is blue
   
       SEE ALSO:color
    */
{
    if(is_void(phase))
        phase = 0.0;
  
    if(is_void(intensite))
        intensite = 1.0;
  
    if(numberof(where(intensite>1)) != 0)
        intensite(where(intensite>1)) = 0;
    
    col = array(double,3,dimsof(phase));

    phi = 6*(phase%(2*pi))/(2*pi);
  
    col(1,) = 255 * intensite * ((phi <= 1.5) * (phi >= 0.) * 1.0 +
                                 (phi <= 3.) * (phi >= 1.5) * (3. - phi)/1.5 +
                                 (phi <= 6.) * (phi > 5.) * 1.0 +
                                 (phi <= 5.) * (phi > 4.) * (phi - 4.));
  
    col(2,) = 255 * intensite * ((phi <= 1.5) * phi / 1.5 +
                                 (phi <= 3.) * (phi > 1.5) *  1. +
                                 (phi <= 4.) * (phi > 3.) * (4. - phi));
    
    col(3,) = 255 * intensite * ((phi <= 3.) * (phi > 2.) * (phi - 2) +
                                 (phi <= 5.) * (phi > 3.) * 1.0 +
                                 (phi <= 6.) * (phi > 5.) * (6. - phi));
  
    return int(col);
}

/***************************************************************************/

func amdlibShowTransferFunctionVsTime(minWlen=, maxWlen=, interpolStyle=, interpolParam=, inputDir=, verbose=, plotScience=, dVisFromV2=)
    /* DOCUMENT amdlibShowTransferFunctionVsTime(minWlen, maxWlen, interpolStyle=, interpolParam=, inputDir=, verbose=, plotScience=, dVisFromV2=)

       DESCRIPTION
       Plot the transfer function vs time for a given spectral region

       PARAMETERS
       - minWlen      : Minimum wavelength range for transfer function computation
       - maxWlen      : Maximum of the wavelength range
       - interpolStyle: style of the interpolation function. Should be one of "sliding" (default), "lines", "slines" or "polynom"
       - interpolParam: parameter of the interpolation function (time constant for "sliding", polynom degree for "polynom")
       - inputDir     : input directory. Should contain science data files
       - verbose      : verbosity
       - plotScience  : plot the science data or not (default 1)
       - dVisFromV2   : 

       RESULTS
       a series of plot of the transfer function values (V^2, closure phases)

       EXAMPLES
       // Plot the transfer function vs time for a given spectral region
       // K band, transfer function as a sliging Gaussian-weighted average
       // (default value)
       > amdlibShowTransferFunctionVsTime, minWlen=2.0e-6, maxWlen=2.4e-6, interpolStyle="sliding", interpolParam=1.0;
       // H band, transfer function fitted as a constant
       > amdlibShowTransferFunctionVsTime, minWlen=1.55e-6, maxWlen=1.65e-6, interpolStyle="polynom", interpolParam=0;

       SEE ALSO: amdlibCalibrateOiData, amdlibShowTransferFunctionVsWlen
    */
{
    colors = ["green","blue","red","magenta","cyan","black"];
    /* Select directory containing files to be treated */
    message = "Choose the product directory (OI_DATA)";    
    if (amdlibFileChooser(message, inputDir, 3) == 0)
    {
        yocoError,"'inputDir' not specified correctly.";
        return 0;
    }
        
    if(is_void(minWlen))
    {
        yocoLogWarning,"minWlen not set, setting it to 2 microns";
        minWlen = 2.0e-6; // Start of the K band
    }
    
    if(is_void(maxWlen))
    {
        yocoLogWarning,"maxWlen not set, setting it to 2.4 microns";
        maxWlen = 2.4e-6; // End of the K band
    }
    
    if(is_void(plotScience))
        plotScience=1;

    if(is_void(interpolStyle))
        interpolStyle="sliding";
    
    /*****************************************
     *** Read calibrators data ***************
     *****************************************/
    sci = orig = trans = [];
    amdlibLoadAllOiData, "CALIB_AVERAGED", orig, inputDir=inputDir;
    if(is_void(orig))
    {
        /* check for the directory */
        amdlibGetProductDir, inputDir+"/", origDir, suffix="AVG", up=1;
        amdlibLoadAllOiData, "CALIB_AVERAGED", orig, inputDir=origDir;
    }

    /* Verbose output */
    if(is_void(orig))
        yocoLogInfo,"No calibrators data !";
    else
        orig.time = orig.time/3600.0; // Time in hrs


    /*****************************************
     *** Read transfer function data *********
     *****************************************/
    amdlibLoadAllOiData , "TRANSFER_FUNCTION", trans, inputDir=inputDir, dVisFromV2=dVisFromV2;
    if(is_void(trans))
    {
        transDir = [];
        /* For amdlib3, check for the transfer function directory */
        if ( !amdlibCheckProductDir(transDir, 0, suffix="TRA", up=1, inputDir=inputDir) )
        {
            yocoError, "Could not find directory with the OIDATA_TRA files", , 1;
            return 0;
        }

        /* Load the data */
        amdlibLoadAllOiData , "TRANSFER_FUNCTION", trans,
            inputDir=transDir, dVisFromV2=dVisFromV2;
    }

    if(is_void(trans))
    {
        yocoError,"No transfer function !";
        return 0;
    }

    trans.time = trans.time/3600.0; // Time in hrs


    /*****************************************
     *** Read science data *******************
     *****************************************/
    if(plotScience==1)
        amdlibLoadAllOiData , "SCIENCE_AVERAGED", sci, inputDir=inputDir;

    if(is_void(sci))
    {
        yocoLogInfo,"No found science data !";
        MAIN_TIME  = trans.time;
        MAIN_NAMES = trans.name ;
    }
    else
    {
        sci.time = sci.time/3600.0; // Time in hrs
        MAIN_TIME  = grow(sci.time,trans.time);
        MAIN_NAMES = grow(sci.name,trans.name);
    }

    // Set pretty limits for the transfer function plot
    TIME0    = min(MAIN_TIME);
    TIME1    = max(MAIN_TIME);
    interval = TIME1 - TIME0;
    limMin   = TIME0 - interval/40;
    limMax   = TIME1 + interval/40;
    MAIN_TIME(where(MAIN_TIME==TIME0)) = limMin;
    MAIN_TIME(where(MAIN_TIME==TIME1)) = limMax;
    
    MAIN_TIME2 = yocoStrRemoveMultiple(MAIN_TIME);
    // set error bars width
    size = max(min(MAIN_TIME2(sort(MAIN_TIME2))(dif)),1.0/30.0)/4.0;
    
    N = numberof(trans.time);
    T2 = T2Err = [];
    
    baseMax = closMax = maxL = maxC = 0;
    for(k=1;k<=N;k++)
    {
        t2 = (*trans.vis2(k))(,,1);
        c2 = (*trans.clos(k))(,,1);
        
        nbBases = dimsof(t2)(2);
        nbClos  = dimsof(c2)(2);
        
        if(nbBases>baseMax)
            baseMax = nbBases;
        
        if(nbClos>closMax)
            closMax = nbClos;
    }
    
    // Plot transfer function
    yocoGuiWinKill;
    amdlibGetPreference,"graphicsDpi",dpi;
    yocoNmCreate,1,1,baseMax+closMax,width=638*dpi/75,height=825*dpi/75,
        V = [0.12,0.73,0.1,0.9];
    

    // Scientific data plot
    if(!is_void(sci))
    {
        _amdlibConstructTimeSequence, sci, minWlen, maxWlen,
            SCIV2, SCIV2Err, SCIClos, SCIClosErr, SCIPhi, SCIPhiErr,
            SCIDvis, SCIDvisErr, SCITIME;
        
        if(max(SCIV2+SCIV2Err)>maxL)
        {
            maxL = max(SCIV2+SCIV2Err);
        }
    
        if(max(abs(SCIClos)+abs(SCIClosErr))>maxC)
        {
            maxC = max(abs(SCIClos)+abs(SCIClosErr));
        }
    
        for(iWin=1;iWin<=baseMax;iWin++)
        {
            plsys,iWin;
        
            yocoPlotWithErrBars, SCIV2(iWin,), SCIV2Err(iWin,), SCITIME-limMin,
                color="red",sizebar=size,type="none";
        }
        
        for(iWin=1;iWin<=closMax;iWin++)
        {
            plsys,iWin+baseMax;
        
            yocoPlotWithErrBars,SCIClos(iWin,),SCIClosErr(iWin,),SCITIME-limMin,
                color="red",sizebar=size,type="none";
        }
    }
    
    // Calibrators raw visibilities plot
    if(!is_void(orig))
    {
        _amdlibConstructTimeSequence, orig, minWlen, maxWlen,
            ORIGV2, ORIGV2Err, ORIGClos, ORIGClosErr, ORIGPhi, ORIGPhiErr,
            ORIGDvis, ORIGDvisErr, ORIGTIME;
    
        if(max(ORIGV2+ORIGV2Err)>maxL)
        {
            maxL = max(ORIGV2+ORIGV2Err);
        }
    
        if(max(abs(ORIGClos)+abs(ORIGClosErr))>maxC)
        {
            maxC = max(abs(ORIGClos)+abs(ORIGClosErr));
        }
    
        for(iWin=1;iWin<=baseMax;iWin++)
        {
            plsys,iWin;
        
            yocoPlotWithErrBars, ORIGV2(iWin,), ORIGV2Err(iWin,), ORIGTIME-limMin,
                color=[192,192,192],sizebar=size,type="none";
        }
        
        for(iWin=1;iWin<=closMax;iWin++)
        {
            plsys,iWin+baseMax;
        
            yocoPlotWithErrBars,ORIGClos(iWin,),ORIGClosErr(iWin,),ORIGTIME-limMin,
                color=[192,192,192],sizebar=size,type="none";
        }
    }


    // Transfer function plot
    _amdlibConstructTimeSequence, trans, minWlen, maxWlen,
        T2, T2Err, T2Clos, T2ClosErr, T2Phi, T2PhiErr,
        T2Dvis, T2DvisErr, TTIME;

    nf = numberof(T2(1,));
    //nf = dimsof(T2)(3);

    // Plot visibilities and closure phases
    for(iWin=1;iWin<=baseMax;iWin++)
        /*MAJ : different color for each calibrator star drawn*/
    {
        plsys,iWin;
        
        for(i=1;i<=nf;i++)
        {
            yocoPlotWithErrBars,T2(iWin,i),T2Err(iWin,i),TTIME(i)-limMin,
                //color="black",sizebar=size,type="none";
                color = _amdlibGetColor(1,i/double(nf)*1.5*pi+pi/6),sizebar=size,type="none";
        }
    }
    
    for(iWin=1;iWin<=closMax;iWin++)
    {
        plsys,iWin+baseMax;
        
	for(i=1;i<=nf;i++)
        {
            yocoPlotWithErrBars,T2Clos(iWin,i),T2ClosErr(iWin,i),TTIME(i)-limMin,
                //color="black",sizebar=size,type="none";
                color = _amdlibGetColor(1,i/double(nf)*1.5*pi+pi/6),sizebar=size,type="none";
        }
    }

    // Plot limits
    if(max(T2+T2Err)>maxL)
    {
        maxL = max(T2+T2Err) + 0.1;
    }
    
    if(max(abs(T2Clos)+abs(T2ClosErr))>maxC)
    {
        maxC = max(abs(T2Clos)+abs(T2ClosErr)) * 1.1;
    }
    
    for(iWin=1;iWin<=baseMax;iWin++)
    {
        plsys,iWin;
        amdlibOverPlotLabels, MAIN_NAMES, MAIN_TIME-limMin, plotName=0;
        limits, 0, limMax-limMin, -0.02, maxL;
        xytitles,,"V^2";
    }
    
    for(iWin=1;iWin<=closMax;iWin++)
    {
        plsys,iWin+baseMax;
        amdlibOverPlotLabels, MAIN_NAMES, MAIN_TIME-limMin, plotName=0,
            MIN=-pi, MAX=pi;
        limits, 0, limMax-limMin, -maxC, maxC;
        xytitles,"Time (h) + MJD "+pr1(limMin/24),"Clos. (rad)";
    }
    
    plsys,1;
    port= viewport();
    xmin = port(1);
    xmax = port(2);
    ymin = port(4);
    T = (MAIN_TIME-limMin)/(limMax-limMin)*(xmax-xmin)+xmin;
    plsys,0;
    amdlibOverPlotLabels, MAIN_NAMES, T, yText=ymin+0.01, justify="LH", plotLines=0;

    // Transfer function interpolation and plot
    interpTime = span(limMin, limMax, 100);

    if(interpolStyle=="polynom")
    {
        if(is_void(interpolParam))
            interpolParam = int(sqrt(dimsof(T2)(3)));
    }
    else if(interpolStyle=="sliding")
    {
        if(is_void(interpolParam))
            interpolParam = 1.0;
    }
    else if(interpolStyle=="lines")
    {}
    else if(interpolStyle=="spline")
    {}
    
    _amdlibInterpTransFunc, T2, T2Err, T2Clos, T2ClosErr,
        T2Phi, T2PhiErr, T2Dvis, T2DvisErr, TTIME, interpTime,
        interpolStyle, interpolParam, interpT2, interpT2Sigma2, interpT2Clos,
        interpT2ClosSigma2, interpT2Phi, interpT2PhiSigma2,
        interpT2Dvis, interpT2DvisSigma2;
        
    for(iWin=1;iWin<=baseMax;iWin++)
    {
        plsys,iWin;
        plg, interpT2(iWin,), interpTime-limMin,
            color="black";
        plg, interpT2(iWin,)+interpT2Sigma2(iWin,), interpTime-limMin,
            color="black",type="dash";
        plg, interpT2(iWin,)-interpT2Sigma2(iWin,), interpTime-limMin,
            color="black",type="dash";
    }
    
    for(iWin=1;iWin<=closMax;iWin++)
    {
        plsys,iWin+baseMax;
        plg, interpT2Clos(iWin,), interpTime-limMin,
            color="black";
        plg, interpT2Clos(iWin,)+interpT2ClosSigma2(iWin,),
            interpTime-limMin, color="black",type="dash";
        plg, interpT2Clos(iWin,)-interpT2ClosSigma2(iWin,), interpTime-limMin,
            color="black",type="dash";
    }
    
    hcps,"./TRANSFER_FUNCTION_TIME_"+pr1(minWlen)+"-"+pr1(maxWlen)+"_Degree_"+pr1(polDeg)+".ps";

}

amdlibShowTransFuncTime = amdlibShowTransferFunctionVsTime;

/***************************************************************************/

func amdlibCalibrateOiData(inputDir=, outputDir=, verbose=, overwrite=, interpolStyle=, interpolParam=, dVisFromV2=)
    /* DOCUMENT amdlibCalibrateOiData(inputDir=, outputDir=, verbose=, overwrite=, interpolStyle=, interpolParam=, dVisFromV2=)

       DESCRIPTION
       V^2 calibration routine. Uses as input a full night OI DATA averaged
       files, and returns as output calibrated files. To check the validity
       of the transfer function, please use amdlibPlotTransferFunctionVsTime
       or amdlibPlotTransferFunctionVsWlen

       PARAMETERS
       - inputDir     : the directory of OI_FITS files where a transfer function
                        has been calculated (with
                        amdlibComputeAllTransferFunction or another).
       - outputDir    : 
       - verbose      : wether verbose mode or not
       - overwrite    : wether existing files are overwritten or not.
       - interpolStyle: Should be any of "sliding", "polynom", "spline",
                        for a Gaussian-weighted average, a polynom or a simple
                        spline interpolation
       - interpolParam: the typical parameter for a given interpolStyle: is a
                        timescale for "sliding", the polynom degree for
                        "polynom", and nothing for "spline"
       - dVisFromV2   : Whether the differential visibility should be extracted
                        directly from the squared visibility

       RESULTS
       A series of calibrated OI data files

       EXAMPLES
       type in a yorick prompt
       // To calibrate with a Sliding average with Gaussian weights
       // (default scheme) with a timescale of 1 hr
       > amdlibCalibrateOiData ,interpolStyle="sliding", interpolParam=1.0;
       // Average over the whole night
       > amdlibCalibrateOiData ,interpolStyle="polynom", interpolParam=0;
       // Linear fit
       > amdlibCalibrateOiData ,interpolStyle="polynom", interpolParam=1;
       // 5th degree polynom
       > amdlibCalibrateOiData ,interpolStyle="polynom", interpolParam=5;
       // Spline interpolation
       > amdlibCalibrateOiData ,interpolStyle="spline";
       // Linear interpolation
       > amdlibCalibrateOiData ,interpolStyle, "lines";
    

       SEE ALSO: amdlibPlotTransferFunctionVsTime,
       amdlibPlotTransferFunctionVsWlen
    */
{

    if(is_void(interpolStyle))
        interpolStyle="sliding";
    
    local sciDir;
  
    /* Select directory containing files to be treated */
    message = "Choose the product directory (OIDATA_AVG)";
    if (amdlibFileChooser(message, inputDir, 3) == 0)
    {
        yocoError, "'inputDir' not specified correctly";
        return 0;
    }
    
    /* Verbose output */
    yocoLogInfo, "Calibrating science data for directory"+
        " (overwrite="+pr1(overwrite)+") :", inputDir;

    /* Check outputDir */
    if ( !amdlibCheckProductDir(outputDir, 1, overwrite, suffix="PRO", up=1, inputDir=inputDir) )
    {
        yocoError, "Could not construct 'outputDir'";
        return 0;
    }

    /* Verbose output */
    yocoLogInfo, "Product directory is:", outputDir;

    /* If existing but overwrite is void, ask the user */
    if (is_void(overwrite))
    {
        overwrite = yocoGuiInfoBox("Directory " +
                                   yocoStrrtok(outputDir,"/")(2) + 
                                   " already exists. Re-calibrate all science data ?\n",
                                   butText=["YES","NO!"],butReturn=[1,0],
                                   dpi=graphicsDpi);
    }
    
    /*****************************************
     *** Read transfer function data *********
     *****************************************/
    amdlibLoadAllOiData , "TRANSFER_FUNCTION", trans, inputDir=inputDir, dVisFromV2=dVisFromV2;
    if(is_void(trans))
    {
        transDir = [];
        /* For amdlib3, check for the transfer function directory */
        if ( !amdlibCheckProductDir(transDir, 0, suffix="TRA", up=1, inputDir=inputDir) )
        {
            yocoError, "Could not find directory with the OIDATA_TRA files", , 1;
            return 0;
        }

        /* Load the data */
        amdlibLoadAllOiData , "TRANSFER_FUNCTION", trans,
            inputDir=transDir, dVisFromV2=dVisFromV2;
    }

    /*****************************************
     *** Read science data *******************
     *****************************************/
    amdlibLoadAllOiData , "SCIENCE_AVERAGED", sci, inputDir=inputDir, dVisFromV2=dVisFromV2;

    
    /* If still void, stop */
    if( is_void(trans) )
    {
        yocoError,"No transfer function files (OIDATA_TRA) !";
        return 0;
    }

    /* If still void, stop */
    if( is_void(sci) )
    {
        yocoError,"No science data files (OIDATA_AVG) !";
        return 0;
    }


    /* Time in hours */
    trans.time = trans.time / 3600.0;
    sci.time   = sci.time   / 3600.0;
    
    N1 = numberof(sci.time);
    N2 = numberof(trans.time);

    CAL_VIS2 = CAL_VIS2_ERR = CAL_CLOS = CAL_CLOS_ERR = [];
    CAL_WLEN = CAL_NAMES = CAL_PHI = CAL_PHI_ERR = [];

    /* Loop on files */
    for(kFile=1;kFile<=N1;kFile++)
    {
        yocoLogInfo,"amdlibCalibrateOiData is now working on file:",sci.name(kFile);

        // Get useful data from science star
        sci_wlen = *sci.wlen(kFile);
        sci_bandwidth = *sci.band(kFile);
        
        v2Val    = (*sci.vis2(kFile))(,,1);
        v2Sigma2 = (*sci.vis2Err(kFile))(,,1)^2;
        
        closVal    = (*sci.clos(kFile))(,,1);
        closSigma2 = (*sci.closErr(kFile))(,,1)^2;

        if(dVisFromV2==1)
        {
            dvisVal    = (*sci.dVis(kFile))(,,1);
            dvisSigma2 = (*sci.dVisErr(kFile))(,,1)^2;
        }
        
        phiVal    = (*sci.dPhi(kFile))(,,1);
        phiSigma2 = (*sci.dPhiErr(kFile))(,,1)^2;
        
        STIME = (sci.time)(kFile);
        
        calV2 = calV2Err = calDvis = calDvisErr = calClos = calClosErr =
            calPhi = calPhiErr = calWlen = calBW = [];

        /* Loop on wavelength bins */
        for(iWlen=1;iWlen<=numberof(sci_wlen);iWlen++)
        {
            minWlen = sci_wlen(iWlen) - sci_bandwidth(iWlen) / 1.5;
            maxWlen = sci_wlen(iWlen) + sci_bandwidth(iWlen) / 1.5;
            
            _amdlibConstructTimeSequence,
                trans, minWlen, maxWlen,
                T2, T2Err, T2Clos, T2ClosErr, T2Phi, 
                T2PhiErr, T2Dvis, T2DvisErr, TTIME;

            // Transfer function interpolation at exact time of observation
            interpTime = STIME;
            
            if(interpolStyle=="polynom")
            {
                if(is_void(interpolParam))
                    interpolParam = int(sqrt(dimsof(T2)(3)));
            }
            else if(interpolStyle=="sliding")
            {
                if(is_void(interpolParam))
                    interpolParam = 1.0;
            }
            else if(interpolStyle=="lines")
            {}
            else if(interpolStyle=="spline")
            {}
                
            _amdlibInterpTransFunc, T2, T2Err, T2Clos, T2ClosErr,
                T2Phi, T2PhiErr, T2Dvis, T2DvisErr, TTIME, interpTime,
                interpolStyle, interpolParam, sT2, sT2Err,
                sT2Clos, sT2ClosErr,
                sT2Phi, sT2PhiErr,
                sT2Dvis, sT2DvisErr;

            
            if(!is_void(T2))
            {
                if(allof(sT2 == 0))
                {
                    yocoError, "Transfer function equal to zero !!!",
                        "Recompute the transfer function with amdlibComputeAllTransferFunction";
                    continue;
                }

                // Squared visibility calibration
                grow, calV2, v2Val(,iWlen)/sT2;
                /*MAJ : des variables sigma2 etaient en fait des erreurs.*/
                grow, calV2Err,
                    sqrt((v2Sigma2(,iWlen)*sT2^2 +
                          sT2Err^2*v2Val(,iWlen)^2)/(sT2^4));
                
                // Closure phase calibration
                grow,calClos,closVal(,iWlen) - sT2Clos;
                grow,calClosErr,
                    sqrt( closSigma2(,iWlen) + sT2ClosErr^2);
                
                // Differential phase calibration
                grow,calPhi,phiVal(,iWlen) - sT2Phi;
                grow,calPhiErr,
                    sqrt( phiSigma2(,iWlen) + sT2PhiErr^2);
                
                if(dVisFromV2)
                {
                    // Differential visibility calibration
                    grow,calDvis,dvisVal(,iWlen) / sT2Dvis;
                    grow,calDvisErr,
                        sqrt(dvisSigma2(,iWlen) + sT2DvisErr^2);
                }
                
                grow,calWlen, sci_wlen(iWlen);
                grow,calBW, sci_bandwidth(iWlen);
            }
        }

        if(numberof(calV2)==0)
        {
            yocoLogWarning,"No calibration data for the star "+sci.name(kFile);
        }
        else
        {
            /* Reading the science file to propagate the input OI data */
            _amdlibReadVis, inoiWave, inamberPhot, inoiVis, inoiVis2, inoiVis3,
                inamberOpd, inamberInsCfg, inoiArray, inoiTarget,
                inputOiFile=sci.file(kFile);
    
            MJD      = (*inoiVis.table).dateObsMJD;
            targetId = (*inoiVis.table).targetId;
            time     = (*inoiVis.table).time;
            DIT      = (*inoiVis.table).expTime;
            stationS = (*inoiVis.table).stationIndex;
        
            nbBases  = dimsof(calV2)(2);
            nbWlen   = dimsof(calV2)(3);
            nbFrames = 1;
        
            if (nbBases == 3)
            {
                stationS3 = (*inoiVis3.table).stationIndex;
            }
        
            amdlibLoadOiData(rwlen,rbandWidth,rtime,rfringeSNR,rsqVis,rsqVisErr,
                             ruvCoord,rbandFlag,rpistonOPDArray,rsigmaPistonArray,rdiffVisAmp,
                             rdiffVisAmpErr,rdiffVisPhase,rdiffVisPhaseErr,rvis3Amp,rvis3AmpErr,rvis3Phase,
                             rvis3PhaseErr,rcpxVis,rcpxVisErr,rvisCovRI,ruv1Coord,ruv2Coord,rfluxSumPiPj,
                             rfluxRatPiPj,rfluxProdPiPj,rspectrum,rspectrumErr,inputOiFile=sci.file(kFile));
        
            U = ruvCoord.u;
            V = ruvCoord.v;
            if (nbBases == 3)
            {
                U1 = transpose(array(ruv1Coord.u,1));
                V1 = transpose(array(ruv1Coord.v,1));
                U2 = transpose(array(ruv2Coord.u,1));
                V2 = transpose(array(ruv2Coord.v,1));
            }

            date = yocoStrReplace(
                                  _amdlibGetKwdVals(sci.file(kFile),
                                                    "DATE-OBS", gui=0)(1),
                                  [":"],["_"]);

            // Setting the output file name
            outName = outputDir + "PRODUCT_" + sci.name(kFile)+"_"+
                strpart(pr1(min(calWlen)),1:4)+"-"+
                strpart(pr1(max(calWlen)),1:4)+"micron_"+date+".fits";

            // Dummy arrays to fill unused tables with zeroes
            cpxDummyArray = array(complex,nbBases,nbWlen,nbFrames);
            dummyArray = array(0.0,nbBases,nbWlen,nbFrames);
            dummy3 = array(0.0,1,nbWlen,1);
        
            /* Check if produced file already exist if yes,
               skip this file expect if overwrite==1 */
            if ( yocoTypeIsFile(outName) && !overwrite )
            {
                yocoLogInfo, "Skipped because the following file already exist:", outName;
                continue;
            }
            else if(yocoTypeIsFile(outName) && overwrite)
            {
                yocoLogInfo, "File " + outName + " is in the way, removing it";
                remove, outName;
            }
            
            amdlibSetWave, newWave, calWlen*yocoAstroSI.m/yocoAstroSI.nm, calBW*yocoAstroSI.m/yocoAstroSI.nm;

            /* MAJ : Add tables containing the differential visibilities */
            /* MAJ : add a condition to manage calibration with and without
               diff. visibility */
            if(dVisFromV2 == 1)
            {
                amdlibSetOiVis, newVis,
                    cpxDummyArray,cpxDummyArray,dummyArray,array(calDvis,1),array(calDvisErr,1), 
                    array(calPhi,1), array(calPhiErr,1),
                    array(0.0,nbBases,nbFrames),
                    targetId, time, MJD, DIT, U, V, stationS;
            }
            else
            {
                amdlibSetOiVis, newVis,
                    cpxDummyArray,cpxDummyArray,dummyArray,dummyArray,dummyArray, 
                    array(calPhi,1), array(calPhiErr,1),
                    array(0.0,nbBases,nbFrames),
                    targetId, time, MJD, DIT, U, V, stationS;
            }
        
            amdlibSetOiVis2, newVis2, array(calV2,1), array(calV2Err,1),
                targetId, time, MJD, DIT, U, V, stationS;
        
            if (nbBases == 3)
            {
                amdlibSetOiVis3, newTriple3,
                    dummy3, dummy3,
                    array(calClos,1),
                    array(calClosErr,1),
                    targetId, time, MJD, DIT, U1, U2, V1, V2, stationS3;
            }
            
            write, "writing file "+outName;
            status = amdlibWriteOiFile(outName,
                                       pointer(nil), &inoiArray, 
                                       &inoiTarget, &newWave, &newPhot,
                                       &newVis, &newVis2, &newTriple3,
                                       &newPiston, pointer(nil));
        
            /* Copy all keywords that could be useful for use afterwards */
            if(_amdlibDumpAllKeywords( 1, sci.file(kFile), outName) == 0)
                yocoError, "Could not write OI files", errMsg, 3;
        
            //             amdlibDumpBinTable, ["OI_ARRAY","OI_TARGET"],
            //                 sci.file(k), outName;
        
            outFh = cfitsio_open(outName, "a", overwrite=1);
            cfitsio_goto_hdu,outFh,1;
            cfitsio_set, outFh, "HIERARCH ESO PRO CATG", "SCIENCE_CALIBRATED",
                "By amdlibCalibrateOiData";
            cfitsio_close_file, outFh;

            grow,CAL_VIS2,&calV2;
            grow,CAL_VIS2_ERR,&calV2Err;
        
            grow,CAL_CLOS,&calClos;
            grow,CAL_CLOS_ERR,&calClosErr;

            grow,CAL_PHI,&calPhi;
            grow,CAL_PHI_ERR,&calPhiErr;

            grow,CAL_WLEN,&calWlen;
            grow,CAL_NAMES,sci.name(kFile);
            grow,CAL_TIME,sci.time(kFile);
        }
    }
}

/***************************************************************************/
      
func _amdlibComputeDiffVisFromV2(&oidataStruct)
/* DOCUMENT _amdlibComputeDiffVisFromV2(&oidataStruct)

   DESCRIPTION
   Computes Differential visibilities from squared visibilities
   
   PARAMETERS
   - oidataStruct: 

   RETURN VALUES

   CAUTIONS

   EXAMPLES

   SEE ALSO
*/
{
    N = numberof(oidataStruct);
    for(k=1;k<=N;k++)
    {
        v2    = *(oidataStruct(k).vis2);
        v2err = *(oidataStruct(k).vis2Err);

        V    = yocoMathSignedSqrt(v2);
        Verr = v2err / (2 * V);

        Vdif    = V    / V(,avg,)(,-,);
        VdifErr = Verr / V(,avg,)(,-,);

        oidataStruct(k).dVis    = &Vdif; 
        oidataStruct(k).dVisErr = &VdifErr; 
    }
}

/***************************************************************************/
      
func _amdlibConstructTimeSequence(transStruct, minWlen, maxWlen, &T2, &T2Err, &T2Clos, &T2ClosErr, &T2Phi, &T2PhiErr, &T2Dvis, &T2DvisErr, &TTIME)
    /* DOCUMENT _amdlibConstructTimeSequence(transStruct, minWlen, maxWlen, &T2, &T2Err, &T2Clos, &T2ClosErr, &T2Phi, &T2PhiErr, &T2Dvis, &T2DvisErr, &TTIME)

       DESCRIPTION
       Build yorick arrays of time sequences of all measurements.
       
       PARAMETERS
       - transStruct: 
       - minWlen    : 
       - maxWlen    : 
       - T2         : 
       - T2Err      : 
       - T2Clos     : 
       - T2ClosErr  : 
       - T2Phi      : 
       - T2PhiErr   : 
       - T2Dvis     : 
       - T2DvisErr  : 
       - TTIME      : 

       RETURN VALUES

       CAUTIONS

       EXAMPLES

       SEE ALSO
    */
{
    N = numberof(transStruct);

    T2 = T2Dvis = T2DvisErr = T2Err = T2Clos = T2ClosErr =
        T2Phi = T2PhiErr = TTIME = [];
    
    for(l=1;l<=N;l++)
    {
        T_wlen = *transStruct.wlen(l);
        
        T_v2 = (*transStruct.vis2(l))(,,1);
        T_v2Err = (*transStruct.vis2Err(l))(,,1);

        if(!is_void((*transStruct.dVis(l))))
            T_dv = (*transStruct.dVis(l))(,,1);
        if(!is_void((*transStruct.dVisErr(l))))
            T_dvErr = (*transStruct.dVisErr(l))(,,1);
        
        T_clos = (*transStruct.clos(l))(,,1);
        T_closErr = (*transStruct.closErr(l))(,,1);
        
        T_phi = (*transStruct.dPhi(l))(,,1);
        T_phiErr = (*transStruct.dPhiErr(l))(,,1); 
        
        idx = where((minWlen<T_wlen)&
                    (maxWlen>T_wlen));
        
        if(numberof(idx)>=1)
        {
            grow,T2,   array(T_v2(,idx)(,avg),1);
            grow,T2Err,array(T_v2Err(,idx)(,avg),1);
            
            if(!is_void((*transStruct.dVis(l))))
                grow,T2Dvis,   array(T_dv(,idx)(,avg),1);
            if(!is_void((*transStruct.dVisErr(l))))
                grow,T2DvisErr,array(T_dvErr(,idx)(,avg),1);
            
            grow,T2Clos,   array(T_clos(,idx)(,avg),1);
            grow,T2ClosErr,array(T_closErr(,idx)(,avg),1);
            
            grow, T2Phi,    array(T_phi(,idx)(,avg),1);
            grow, T2PhiErr, array(T_phiErr(,idx)(,avg),1);
            
            grow, TTIME, transStruct.time(l);
        }
    }
    
}

/***************************************************************************/
      
func _amdlibInterpTransFunc(T2, T2Err, T2Clos, T2ClosErr, T2Phi, T2PhiErr, T2Dvis, T2DvisErr, TTIME, interpTime, interpFunc, interpParam, &interpT2, &interpT2Sigma2, &interpT2Clos, &interpT2ClosSigma2, &interpT2Phi, &interpT2PhiSigma2, &interpT2Dvis, &interpT2DvisSigma2)
    /* DOCUMENT _amdlibInterpTransFunc(T2, T2Err, T2Clos, T2ClosErr, T2Phi, T2PhiErr, T2Dvis, T2DvisErr, TTIME, interpTime, interpFunc, interpParam, &interpT2, &interpT2Sigma2, &interpT2Clos, &interpT2ClosSigma2, &interpT2Phi, &interpT2PhiSigma2, &interpT2Dvis, &interpT2DvisSigma2)

       DESCRIPTION
       Interpolate the transfer function at the science data times

       PARAMETERS
       - T2                : 
       - T2Err             : 
       - T2Clos            : 
       - T2ClosErr         : 
       - T2Phi             : 
       - T2PhiErr          : 
       - T2Dvis            : 
       - T2DvisErr         : 
       - TTIME             : 
       - interpTime        : 
       - interpFunc        : 
       - interpParam       : 
       - interpT2          : 
       - interpT2Sigma2    : 
       - interpT2Clos      : 
       - interpT2ClosSigma2: 
       - interpT2Phi       : 
       - interpT2PhiSigma2 : 
       - interpT2Dvis      : 
       - interpT2DvisSigma2: 

       RETURN VALUES

       CAUTIONS

       EXAMPLES

       SEE ALSO
    */
{
    interpT2=interpT2Sigma2=interpT2Clos=interpT2ClosSigma2=interpT2Phi=interpT2PhiSigma2=interpT2Dvis=interpT2DvisSigma2=[];

    if(interpFunc=="polynom")
        interpFunc = _amdlibInterpFuncPoly;
    else if(interpFunc=="sliding")
        interpFunc = _amdlibInterpFuncGauss;
    else if(interpFunc=="lines")
        interpFunc = _amdlibInterpFuncLines;
    else if(interpFunc=="spline")
        interpFunc = _amdlibInterpFuncSplines;
    else
        error, "interpolation function not valid. Should be one of \"polynom\", \"sliding\", \"lines\", or \"spline\"";

    // If no transfer function then exit.
    if(numberof(TTIME)==0)
        return 0;
    
    // If only one transfer function point, then no interpolation /*MAJ*/
    if(numberof(TTIME)==1)
    {
        interpT2 = array(T2(,1), numberof(interpTime));
        interpT2Sigma2 = array(T2Err(,1), numberof(interpTime));
        interpT2Clos = array(T2Clos(,1), numberof(interpTime));
        interpT2ClosSigma2 = array(T2ClosErr(,1), numberof(interpTime));
        interpT2Phi = array(T2Phi(,1), numberof(interpTime));
        interpT2PhiSigma2 = array(T2PhiErr(,1), numberof(interpTime));
        interpT2Dvis = T2Dvis;
        interpT2DvisSigma2 = T2DvisErr;
    
        return 1; 
    }
    
    minTime = min(TTIME);
    TTIME = TTIME - minTime;
    interpTime = interpTime - minTime;

    // Compute useful dimensions
    nbBases = dimsof(T2)(2);
    nbClos  = dimsof(T2Clos)(2);
    nbObs   = numberof(interpTime);
    
    // Initialize output arrays
    interpT2 = interpT2Sigma2 =
        interpT2Phi = interpT2PhiSigma2 =
        interpT2Dvis = interpT2DvisSigma2 = array(0.0, nbBases, nbObs);
    interpT2Clos = interpT2ClosSigma2 = array(0.0, nbClos, nbObs);
    
    // Compute the smoothed transfer function
    for(iBase=1; iBase <= nbBases; iBase++)
    {
        interpFunc, T2(iBase,), T2Err(iBase,),
            TTIME, interpTime, interpParam,
            tmpT2, tmpT2Sigma2;
        
        interpT2(iBase,)       = tmpT2;
        interpT2Sigma2(iBase,) = tmpT2Sigma2;
        
        interpFunc, T2Phi(iBase,), T2PhiErr(iBase,),
            TTIME, interpTime, interpParam,
            tmpT2Phi, tmpT2PhiSigma2;
        
        interpT2Phi(iBase,)       = tmpT2Phi;
        interpT2PhiSigma2(iBase,) = tmpT2PhiSigma2;

        if(!is_void(T2Dvis))
        {
            interpFunc, T2Dvis(iBase,), T2DvisErr(iBase,),
                TTIME, interpTime, interpParam,
                tmpT2Dvis, tmpT2DvisSigma2;
            
            interpT2Dvis(iBase,)       = tmpT2Dvis;
            interpT2DvisSigma2(iBase,) = tmpT2DvisSigma2;
        }
    }
    
    // Compute the smoothed transfer function
    for(iClos=1; iClos <= nbClos; iClos++)
    {
        
        interpFunc, T2Clos(iClos,), T2ClosErr(iClos,),
            TTIME, interpTime, interpParam,
            tmpT2Clos, tmpT2ClosSigma2;
        
        interpT2Clos(iClos,)       = tmpT2Clos;
        interpT2ClosSigma2(iClos,) = tmpT2ClosSigma2;
    }
}

/***************************************************************************/

func _amdlibInterpFuncPoly(vals, valsErr, valsTime, interpTime, polDeg, &interpVals, &interpValsSigma2)
    /* DOCUMENT _amdlibInterpFuncPoly(vals, valsErr, valsTime, interpTime, polDeg, &interpVals, &interpValsSigma2)

       DESCRIPTION
       Interpolation function using polynoms

       PARAMETERS
       - vals            : 
       - valsErr         : 
       - valsTime        : 
       - interpTime      : 
       - polDeg          : 
       - interpVals      : 
       - interpValsSigma2: 

       RETURN VALUES

       CAUTIONS

       EXAMPLES

       SEE ALSO
    */
{
    // Specific case for a zero degree polynom: take the average
    if(polDeg==0)
    {
        if(numberof(valsTime)==1)
        {
            interpVals       = vals;
            interpValsSigma2 = valsErr^2;
        }
        else
        {
            interpVals       = vals(avg);
            interpValsSigma2 = (vals(rms)^2 + valsErr(avg)^2)/2;
        }
    }
    else
    {
        // Fit the transfer function with a polynom
        coefVals = yocoMathPolyFit(polDeg, valsTime-avg(valsTime), vals);
        polyVals = yocoMathPoly(valsTime-avg(valsTime), coefVals);
            
        // Compute dispersion of Vals around the polynom
        rmsVals = sqrt(sum((vals - polyVals)^2)/
                       numberof(vals));
            
        // Interpolate the transfer function
        interpVals       = yocoMathPoly(interpTime-avg(valsTime), coefVals);
        interpValsSigma2 = sqrt(rmsVals^2 + valsErr(avg)^2);
    }
}

/***************************************************************************/

func _amdlibInterpFuncGauss(vals, valsErr, valsTime, interpTime, timeConstant, &interpVals, &interpValsSigma2)
    /* DOCUMENT _amdlibInterpFuncGauss(vals, valsErr, valsTime, interpTime, timeConstant, &interpVals, &interpValsSigma2)

       DESCRIPTION
       interpolation function using a Gaussianb-weighted average

       PARAMETERS
       - vals            : 
       - valsErr         : 
       - valsTime        : 
       - interpTime      : 
       - timeConstant    : 
       - interpVals      : 
       - interpValsSigma2: 

       RETURN VALUES

       CAUTIONS

       EXAMPLES

       SEE ALSO
    */
{
    atVal = itpVal = [];


    FWHM_factor = 2 * sqrt(2*log(2));
    
    N2 = numberof(interpTime);
    for(k=1;k<=N2;k++)
    {
        weight = (timeConstant/FWHM_factor * sqrt(2 * pi)) *
            yocoMathGauss(valsTime, [timeConstant/FWHM_factor, interpTime(k)]);
        if(allof(weight==0))
            return 0;
        grow, itpVal, sum(vals * weight) / sum(weight);
    }
    
    N1 = numberof(valsTime);
    for(k=1;k<=N1;k++)
    {
        weight = (timeConstant/FWHM_factor * sqrt(2 * pi)) *
            yocoMathGauss(valsTime, [timeConstant/FWHM_factor, valsTime(k)]);
        grow, atVal, sum(vals * weight) / sum(weight);
    }
    
    // Compute dispersion of Vals around the transfer function
    rmsVals = sqrt(sum((vals - atVal)^2) /
                   numberof(vals));

    interpVals = itpVal;
    interpValsSigma2 = sqrt(rmsVals^2 + valsErr(avg)^2);
}

/***************************************************************************/
      
func _amdlibInterpFuncLines(vals, valsErr, valsTime, interpTime, timeConstant, &interpVals, &interpValsSigma2)
    /* DOCUMENT _amdlibInterpFuncLines(vals, valsErr, valsTime, interpTime, timeConstant, &interpVals, &interpValsSigma2)

       DESCRIPTION
       interpolation function with lines

       PARAMETERS
       - vals            : 
       - valsErr         : 
       - valsTime        : 
       - interpTime      : 
       - timeConstant    : 
       - interpVals      : 
       - interpValsSigma2: 

       RETURN VALUES

       CAUTIONS

       EXAMPLES

       SEE ALSO
    */
{
    interpVals = interp(vals, valsTime, interpTime);
}

/***************************************************************************/
      
func _amdlibInterpFuncSplines(vals, valsErr, valsTime, interpTime, timeConstant, &interpVals, &interpValsSigma2)
    /* DOCUMENT _amdlibInterpFuncSplines(vals, valsErr, valsTime, interpTime, timeConstant, &interpVals, &interpValsSigma2)

       DESCRIPTION
       Interpolation function with splines

       PARAMETERS
       - vals            : 
       - valsErr         : 
       - valsTime        : 
       - interpTime      : 
       - timeConstant    : 
       - interpVals      : 
       - interpValsSigma2: 

       RETURN VALUES

       CAUTIONS

       EXAMPLES

       SEE ALSO
    */
{
    interpVals = spline(vals, valsTime, interpTime);
}


/* ******************************************************************
   Calibration routines based on oiFitsUtils.i
   ****************************************************************** */


func amdlibCalibrateAllOiData(inputDir=, outputDir=, keepTime=, keepWavelength=, calibTextFile=, overwrite=, plotWavelength=, checkPlot=, vis2TfMode=, t3TfMode=, visTfMode=, groupOiData=, removeTime=,writeOutputFiles=,normalizeContinuum=, funcSetup=, minVis2Err=, minCloPhiErr=, dontUseCalibrationStars=)
/* DOCUMENT amdlibCalibrateAllOiData(inputDir=, outputDir=, keepTime=, keepWavelength=,
                                 removeTime=,
                                 calibTextFile=, overwrite=, checkPlot=,
                                 vis2TfMode=, t3TfMode=, visTfMode=,
                                 groupOiData=, plotWavelength=,
                                 writeOutputFiles=,
                                 normalizeContinuum=,
                                 funcSetup=, minVis2Err=, minPhiCloErr=)

   DESCRIPTION
   Perform the interferometric calibration of the OIDATA_AVG.
   Should be run in a directory that contains both the CALIB_AVERAGED and
   the SCIENCE_AVERAGED data. Results are stored as SCIENCE_CALIBRATED product,
   and are called *OIDATA_CAL.fits

   The diameters of the calibration stars should be provided in the file
   calibTextFile (by the mean of amdlibSearchAllStarDiameters or
   amdlibQueryAllStarDiameters).

   The optional parameters allow to adapt the the calibration process in a
   semi-interactive way: remove some periods of time, remove some wavelength
   ranges, plot results, store the product. See the example.

   PARAMETERS
   - inputDir        : directory containing the input OIDATA_AVG
   - outputDir       : directory for the output OIDATA_CAL product.
   - overwrite       : 0/1
   - calibTextFile   : ASCII file that contains the diameters of the calibration
                       stars. Can be created by amdlibSearchAllStarDiameters.
   
   Simple options to adapt the calibration strategy:
   -------------------------------------------------
                       
   - checkPlot       : is set to 1, two plot of the transfer function are provided.
                       - versus time (averaged over plotWavelength)
                       - versus lambda
                       
   - plotWavelength  : average over this wavelegth range for the plot
                       [start,end] or [[start1,end1],...,[startN, endN]].

   - writeOutputFiles: 0/1, default is 0 (output OIDATA_AVG files are not saved).

   - keepTime        : optional parameter to keep only a certain part of
                       the data. Should be specified in MJD and be of the form:
                       [start,end] or [[start1,end1],...,[startN, endN]].
                       
   - removeTime      : same as keepTime (but remove these time intervals).
                     
   - keepWavelength  : optional parameter to keep only a certain
                       part of the data. Should be specified in mum (10-6m) and be
                       [start,end] or [[start1,end1],...,[startN, endN]].

   Advance-user options to adapt the calibration strategy:
   -------------------------------------------------------
                     
   - xxxTfMode       : methode to interpolate the transfer function.
                       Can be xxxtfMode=
                       0: interpolate the 2 closest TF-points
                       1: smooth-length (>1pt)
                       2: average       (>1pt)  (default)
                       3: linear fit    (>2pt)
                       4: quadratic fit (>3pt)

   - minVis2Err      : force the relative uncertainity of the vis2 to be
                       larger than this value (useful for highly unstable
                       transfer function). Example for 10%: minVis2Err=0.1;
   
   - minPhiCloErr    : force the absolute uncertainity of the closure phase
                       to be larger than this value.
                       Example: minCloPhiErr=10deg.
                       
   - normalizeContinuum : force the continuum to be unresolved (V2=1 and phi=0)
                       This normalization is done by fitting a polynomial law
                       in the specified wavelength range (in 10-6m). Example:
                       normalizeContinuum=[[2.1,2.15], [1.15,2.2]];

   - groupOiData     : group the consecutive OIDATA (same setup, same target)
                       into a single measurement.

   - funcSetup       : 
                       
   EXAMPLES
   > // first run, just to see
   > amdlibCalibrateAllOiData, inputDir="./", checkPlot=1;
   > 
   > // inspect the result, decide to remove the few first points
   > amdlibCalibrateAllOiData, inputDir="./", checkPlot=1,
                               keepTime=54689+[0.5,0.78];
   > 
   > // inspect the result, decide to remove the J-band
   > amdlibCalibrateAllOiData, inputDir="./", checkPlot=1,
                               keepTime=54689+[0.5,0.78],
                               keepWavelength[[1.5,1.8],[2.1,3.0]];
   > 
   > // OK, now write the results
   > amdlibCalibrateAllOiData, inputDir="./", writeOutputFiles=1,
                               keepTime=54689+[0.5,0.78],
                               keepWavelength[[1.5,1.8],[2.1,3.0]];
   > 
   
   SEE ALSO
*/
{
    local files,oiTarget,oiWave,oiArray,oiVis2,oiVis,oiT3,oiLog;
    local data, tit, proCatg, id1, id2, id3, lof, i;
    local message, add, order;
    yocoLogInfo,"amdlibCalibrateAllOiData()";

    /* Default  */
    if ( is_void(keepTime) ) keepTime     = [0.,10e6];
    if ( is_void(removeTime) ) removeTime = [-1,0];
    if ( is_void(writeOutputFiles) ) writeOutputFiles = 0;
    if ( is_void(plotWavelength) ) plotWavelength = [2.1,2.3];

    /* Check simples parameters */
    if ( !amdlibCheckOverwrite(overwrite)       ||
         !amdlibCheckGraphicsDpi(graphicsDpi)   ||
         !amdlibCheckCalibTextFile(calibTextFile) ||
         !yocoTypeIsIntegerScalar(writeOutputFiles) ||
         !yocoTypeIsNumerical(keepTime) ||
         !yocoTypeIsNumerical(removeTime) )
    {
        yocoError, "Check function's parameters";
        return 0;
    }


    /* Select directory containing files to be treated */
    message = "Choose the input directory (OIDATA_AVG)";
    if ( !amdlibFileChooser(message, inputDir, 3)  ||
         !amdlibCheckDir(inputDir, 0) )
    {
        yocoError,"'inputDir' not specified correctly.";
        return 0;
    }

    /* Chose the suffix depending on the analysis
       (differential or absolute) */
    suffix = ( is_array(normalizeContinuum) ? "CALDIFF" : "CAL" );

    /* Check outputDir */
    if ( writeOutputFiles &&
         !amdlibCheckProductDir(outputDir, 1, overwrite, suffix=suffix, up=1, inputDir=inputDir) )
    {
        yocoError, "Could not construct 'outputDir'";
        return 0;
    }

    /* Read the log of the inputDir */
    if ( !amdlibReadLogFromDir(inputDir, data, tit) )
    {
        yocoError, "Could not read logfile of 'inputDir'";
        return 0;
    }
  
    /* Find SCIENCE_AVERAGED and CALIB_AVERAGED data
       that are within the MJD interval. Use a large time interval
       because we use the HEADER MJD which is not accurate.
       The precise selection is done latter. */
    proCatg = data( where(tit=="pro_catg")(1), );
    mjdObs  = yocoStr2Double(data( where(tit=="MJD-OBS")(1), ) );
    id      = where(strmatch(proCatg,"_AVERAGED") &
                    oiFitsIsInsideInterv(keepTime+[-0.04,+0.04],mjdObs) &
                    !oiFitsIsInsideInterv(removeTime+[+0.04,-0.04],mjdObs));
  
    /* Check if any */
    if ( numberof(id)<1 )
    {
        yocoError,"Cannot find any OIDATA_AVG in 'inputDir' and within the time interval.";
        return 0;
    }

    /* Build the full path+filenames */
    inputOiFile = data( where(tit=="fileName")(1), id );
    inputOiFile = inputDir + inputOiFile;
  
    /* Load all files  */
    oiFitsLoadFiles,inputOiFile,oiTarget,oiWave,oiArray,oiVis2,oiVis,oiT3,oiLog;

    /* Execute the requested cuts in time and wavelengths */
    oiFitsKeepTime, oiVis2, oiVis, oiT3, tlimit=keepTime;
    oiFitsKeepTime, oiVis2, oiVis, oiT3, tlimit=removeTime, invers=1;
    oiFitsKeepWave, oiWave, oiVis2, oiVis, oiT3, wlimit=keepWavelength;
    oiFitsCleanUnused,oiTarget,oiWave,oiArray,oiVis2,oiVis,oiT3,oiLog;

    /* Load the diameter of calibration stars, or perform a differential analysis
       by normalizing the continuum value */
    if ( is_array(normalizeContinuum) )
    {
        yocoLogInfo,"Differential calibration, diameters of calibration stars will not be used.";
        if ( !_amdlibLoadOiDiamFromTarget( oiTarget, oiLog, oiDiam) )
        {
           yocoError, "Cannot load calibrator information.";
           return 0;
        }
        
        /* Normalize the continnum */
        yocoLogInfo,"Normalize the continnum using: ", pr1(normalizeContinuum);
        oiFitsNormalizeContinuum, oiVis2, oiWave, cont=normalizeContinuum, order=1;
        oiFitsNormalizeContinuum, oiVis,  oiWave, cont=normalizeContinuum, order=1;
        oiFitsNormalizeContinuum, oiT3,   oiWave, cont=normalizeContinuum, order=1;

        /* Chose the appropriate function for the AMBER setup */
        if ( is_void(funcSetup) ) funcSetup = _amdlibAmberSetupDiff;
    
    }
    else if ( dontUseCalibrationStars )
    {
        yocoError, "Option 'dontUseCalibrationStars' can only be used with option 'normalizeContinnuum'";
        return 0;
    }
    else if ( !_amdlibLoadOiDiamFromFile( calibTextFile, oiTarget, oiLog, oiDiam ) )
    {
        yocoError, "Cannot load calibrator diameter from file", calibTextFile;
        return 0;
    }

    /* Chose the appropriate function for the AMBER setup */
    if ( is_void(funcSetup) ) funcSetup = _amdlibAmberSetup;
  
    /* Eventuall group consecutive OIDATA */
    if ( groupOiData )
    {
        yocoLogInfo,"Group consecutives OIDATA";
        oiFitsGroupAllOiData, oiVis2, oiVis, oiT3, oiLog, errMode=5;
    }
  
    /* Now perform the calibration. This suppose the closure phase of the
       calibration star is zero. Note that the oiVisData (cpx) are not
       calibrated nor the AMBER spectrum */

    if ( dontUseCalibrationStars==1 )
    {
        /* This mode may be insteresting when doing differential
           analysis and when the calibration stars are very noisy */
        yocoLogInfo, "You decided to not use calibration stars";
        if (is_array(oiT3))
        {
            oiT3Cal = oiT3( where(oiFitsGetOiDiam(oiT3,oiDiam).isCal==0) );
        }
        if (is_array(oiVis2))
        {
            oiVis2Cal = oiVis2( where(oiFitsGetOiDiam(oiVis2,oiDiam).isCal==0) );
        }
        if (is_array(oiVis))
        {
            oiVisCal = oiVis( where(oiFitsGetOiDiam(oiVis,oiDiam).isCal==0) );
        }
    }
    else
    {
        /* This should be the standar mode, even in differential analysis
           as the calibration stars also remove some instrumental effects.
           Obvisouly, this is mandatory when doing absolute calibration. */
        if (is_array(oiT3))
        {
            oiFitsExtractTf, oiT3, oiWave, oiDiam, oiT3Tfp;
            oiFitsApplyTf, oiT3, oiT3Tfp, oiArray, oiLog, oiT3Cal, oiT3Tfe,
                tfMode=t3TfMode, funcSetup=funcSetup,minPhiErr=minCloPhiErr;
        }
        if (is_array(oiVis2))
        {
            oiFitsExtractTf, oiVis2, oiWave, oiDiam, oiVis2Tfp;
            oiFitsApplyTf, oiVis2, oiVis2Tfp, oiArray, oiLog, oiVis2Cal, oiVis2Tfe,
                tfMode=vis2TfMode, funcSetup=funcSetup, minAmpErr=minVis2Err;
        }
        if (is_array(oiVis))
        {
            oiFitsExtractTf, oiVis, oiWave, oiDiam, oiVisTfp;
            oiFitsApplyTf, oiVis, oiVisTfp, oiArray, oiLog, oiVisCal, oiVisTfe,
                tfMode=visTfMode, funcSetup=funcSetup;
        }
    }

  
    /* Now, eventually plot the results versus time and baselines.
       Only oiVis2 and oiT3 are plotted. */
    if (checkPlot)
    {
        oiFitsGraphicDpi, dpi = graphicsDpi;
        oiFitsPlotCalibratedNight, oiWave, oiArray, oiLog, oiTarget, oiDiam,
            oiVis2, oiVis2Cal, oiVis2Tfe, oiVis2Tfp,
            oiT3, oiT3Cal, oiT3Tfe, oiT3Tfp,
            Avg=plotWavelength, win=checkPlot;
    }

    /* Eventually stop here */
    if ( !writeOutputFiles )
    {
        yocoLogInfo,"OIDATA_CAL files not written (writeOutputFiles=0)."
            yocoLogInfo,"amdlibCalibrateAllOiData done";
        return 1;
    }
    
    /* Loop on files to write them, normaly each written file will have
       3 OIVIS2, 3 OIVIS and 1 OIT3 as this was the cases in the input files.
       However, no check is done regarding this structure. */
  
    /* Find the filename of the scientific files, that is where isCal==0 */
    if ( !is_array( (id=where(oiFitsGetOiDiam( oiVis2, oiDiam ).isCal ==0) )))
    {
        yocoLogInfo,"No scientific files in this directory.";
        return 1;
    }
    
    /* Loop on files */
    listOfFiles = yocoListClean( oiFitsGetOiLog(oiVis2(id),oiLog).fileName );
    for ( i=1 ; i<=numberof(listOfFiles) ; i++ )
    {
        /* Construct output OI file name.
           Use the same rule as amdlibComputeTransferFunction */
        yocoFileSplitName,listOfFiles(i),caldir,calname,calext;
        outputOiFile = outputDir + yocoStrReplace(calname,"AVG",suffix)+calext;

        /* Check if product file already exists */
        if ( yocoTypeIsFile(outputOiFile) && (overwrite==0) )
        {
            yocoLogInfo,"Skipped because the following file already exist:", outputOiFile;
            continue;
        }

        /* Find if CAL data exist for this file */
        id1 = where(oiFitsGetOiLog(oiVis2Cal,oiLog).fileName==listOfFiles(i));
        id2 = where(oiFitsGetOiLog(oiVisCal,oiLog).fileName==listOfFiles(i));
        id3 = where(oiFitsGetOiLog(oiT3Cal,oiLog).fileName==listOfFiles(i));

        oVis2 = oiVis2Cal( id1 );
        oVis  = oiVisCal( id2 );
        oT3   = oiT3Cal( id3 );

        if ( !is_array(id1) && !is_array(id2) && !is_array(id3))
        {
            yocoLogWarning,"No calibrated data for this file:",listOfFiles(i);
            continue;
        }
        
        /* Reorder so that it match the order of baselines in
           uncalibrated data (for amdlibShowOiData to work)*/
        if ( is_array(oVis2) )
        {
          oLog = oiFitsGetOiLog(oVis2(1),oiLog)(1);
          ido1 = where(oiFitsGetBaseName(oVis2,oiArray) == oLog.issStation1+"-"+oLog.issStation2)(1);
          ido2 = where(oiFitsGetBaseName(oVis2,oiArray) == oLog.issStation2+"-"+oLog.issStation3)(1);
          ido3 = where(oiFitsGetBaseName(oVis2,oiArray) == oLog.issStation1+"-"+oLog.issStation3)(1);
          
          /* Re-order*/
          if (is_array(ido1) && is_array(ido2) && is_array(ido3))
             oVis2 = oVis2([ido1,ido2,ido3]);
          else
            yocoLogWarning,"Cannot re-order baselines in OIFITS file product,"+
              "file is OIFITS compliant but may not open properly with 'amdlibShowOiData'";
        }
        
        if ( is_array(oVis) )
        {
          oLog = oiFitsGetOiLog(oVis(1),oiLog)(1);
          ido1 = where(oiFitsGetBaseName(oVis,oiArray) == oLog.issStation1+"-"+oLog.issStation2)(1);
          ido2 = where(oiFitsGetBaseName(oVis,oiArray) == oLog.issStation2+"-"+oLog.issStation3)(1);
          ido3 = where(oiFitsGetBaseName(oVis,oiArray) == oLog.issStation1+"-"+oLog.issStation3)(1);
          /* Re-order*/
          if (is_array(ido1) && is_array(ido2) && is_array(ido3))
            oVis = oVis([ido1,ido2,ido3]);
          else
            yocoLogWarning,"Cannot re-order baselines in OIFITS file product,"+
              "file is OIFITS compliant but may not open properly with 'amdlibShowOiData'";
        }

        
        /* Handle compressed file */
        compress = amdlibCheckCompress(outputOiFile);

        /* Write data */
        oiFitsWriteFile, outputOiFile, oiTarget(*), oiWave(*), oiArray(*),
            oVis2, oVis, oT3, overwrite=1;
        
        /* Dump keywords */
        amdlibCopyHeader, listOfFiles(i), outputOiFile, "HIERARCH ESO PRO CATG", "SCIENCE_CALIBRATED";

        
        /* Read AMBER_SPECTRUM to dump it uncalibrated into the product file.
           Actually we also read the OI_WAVELENGTH table so that we can
           re-project the spectrum in this table, to ensure that all
           previous operation (cut, sort) are propagated to the spectrum */

        /* Read AMBER_SPECTRUM table */
        fh = cfitsio_open(listOfFiles(i));
        cfitsio_goto_hdu,fh,"AMBER_SPECTRUM";
        pcol = cfitsio_read_bintable(fh,titles);
        cfitsio_close,fh;

        /* Open output file and read OI_WAVELENGTH table */
        fh = cfitsio_open(outputOiFile,"a");
        cfitsio_goto_hdu,fh,"OI_WAVELENGTH";
        lbd  = *cfitsio_read_bintable(fh)(1);

        /* Avoid writing back eventual unkown supplementary colums added by Gilles */
        pcolsub=pcol(1:4)
        /* Project AMBER_SPECTRUM into the OI_WAVELENGTH table */
        pcolsub(4) = &( interp(*pcol(4), *pcol(1), lbd, 2) );
        pcolsub(3) = &( interp(*pcol(3), *pcol(1), lbd, 2) );
        pcolsub(2) = &( interp(*pcol(2), *pcol(1), lbd) );
        pcolsub(1) = &lbd;
        
        /* Dump AMBER_SPECTRUM (uncalibrated) */
        cfitsio_add_bintable, fh, pcolsub, titles, ,"AMBER_SPECTRUM";
        cfitsio_close,fh;

        /* Handle compressed file */
        if ( !amdlibCompress(outputOiFile,compress) )
        {
            yocoError, "Could not compress the output OIDATA_CAL file";
            continue;
        }
    
    }
    /* End loop on files */
  
  
    yocoLogInfo,"amdlibCalibrateAllOiData done";
    return 1;
}

/****************************************************************/


func _amdlibAmberSetupDiff(oiData, oiLog)
    /* DOCUMENT _amdlibAmberSetupDiff(oiData, oiLog)

       PRIVATE ROUTINE.

       DESCRIPTION
       Return the AMBER setup used of the observation.
       In this specific case, the setup is the same for oiT3, oiVis and oiVis2.

       The setup is defined by:  insMode, insName, p2vmId

       This is for differential use only.
       Absolute calibration cannot be achieved.

       PARAMETERS
       - oiData: valid oi_structure loaded with oiFitsLoadFiles.
       - oiLog : valid oi_structure loaded with oiFitsLoadFiles.
    */
{
    local setup,dit,log;

    /* Check inputs */
    if( !oiFitsIsOiData(oiData) ) return yocoError("oiData not valid");
    if( !oiFitsIsOiLog(oiLog) )   return yocoError("oiLog not valid");

    /* Recover the log */
    log = oiFitsGetOiLog(oiData,oiLog);

    setup = swrite(format="%s - %s - %d",log.insMode,
                   oiData.hdr.insName,log.p2vmId);
  
    /* return the string */
    return setup;
}

/****************************************************************/

func _amdlibAmberSetup(oiData, oiLog)
    /* DOCUMENT _amdlibAmberSetup(oiData, oiLog)

       PRIVATE ROUTINE.
   
       DESCRIPTION
       Return the AMBER setup used of the observation.
       In this specific case, the setup is the same for oiT3, oiVis and oiVis2.

       The setup is defined by:  dit, ndit, finito-mode (not DIT), insName, p2vmId

       PARAMETERS
       - oiData: valid oi_structure loaded with oiFitsLoadFiles.
       - oiLog : valid oi_structure loaded with oiFitsLoadFiles.
    */
{
    local setup,dit,log;

    /* Check inputs */
    if( !oiFitsIsOiData(oiData) ) return yocoError("oiData not valid");
    if( !oiFitsIsOiLog(oiLog) )   return yocoError("oiLog not valid");

    /* Recover the log */
    log = oiFitsGetOiLog(oiData,oiLog);

    /* TRICK: put the dit close the 25ms at 25ms exactly */
    dit = int( log.dit * 1000.0 );
    dit += ( dit==24 );
    dit -= ( dit==26 );
    dit += ( dit==49 );
    dit -= ( dit==51 );
  
    setup = swrite(format="%s %ix%.3fs - %s %.1fms - %s - %d",log.insMode,
                   log.ndit, dit/1000.0, log.ftSensor+" "+log.opdcAlgoType,
                   log.issFntDitCh1*0.0,oiData.hdr.insName,log.p2vmId);
  
    /* return the string */
    return setup;
}

/****************************************************************/

func _amdlibAmberSetupWithFinitoDit(oiData, oiLog)
{
    local setup,dit,log;

    /* Check inputs */
    if( !oiFitsIsOiData(oiData) ) return yocoError("oiData not valid");
    if( !oiFitsIsOiLog(oiLog) )   return yocoError("oiLog not valid");

    /* Recover the log */
    log = oiFitsGetOiLog(oiData,oiLog);

    /* TRICK: put the dit close the 25ms at 25ms exactly */
    dit = int( log.dit * 1000.0 );
    dit += ( dit==24 );
    dit -= ( dit==26 );
    dit += ( dit==49 );
    dit -= ( dit==51 );
  
    setup = swrite(format="%s %ix%.3fs - %s %.1fms - %s - %d",log.insMode,
                   log.ndit, dit/1000.0, log.ftSensor+" "+log.opdcAlgoType,
                   log.issFntDitCh1*1000.0,oiData.hdr.insName,log.p2vmId);
  
    /* return the string */
    return setup;
}


/* use this as default */
oiFitsDefaultSetup = _amdlibAmberSetup;

/****************************************************************/

func _amdlibLoadOiDiamFromTarget(oiTarget, oiLog, &oiDiam)
    /* DOCUMENT _amdlibLoadOiDiamFromTarget(oiTarget, oiLog, &oiDiam)

       DESCRIPTION
       Create a valid structure oiDiam based on oiTarget and oiLog.
       However, all diameter will be 0.
    */
{
    yocoLogTrace,"_amdlibLoadOiDiamFromTarget()";
    local isCal;
  
    /* Fill the structure */
    oiDiam          = array(struct_oiDiam, numberof(oiTarget) );
    oiDiam.targetId = oiTarget.targetId;
    oiDiam.isCal    = ( oiFitsGetOiLog(oiTarget,oiLog).dprCatg == "CALIB" );

    /* Check if some calibrator are filled */
    if ( allof(oiDiam.isCal==0) )
    {
        yocoLogInfo,"No calibrators have been found.";
    }

    yocoLogTrace,"_amdlibLoadOiDiamFromTarget done";
    return 1;
}

/****************************************************************/

func _amdlibLoadOiDiamFromFile(calibTextFile, oiTarget, oiLog, &oiDiam)
    /* DOCUMENT _amdlibLoadOiDiamFromFile(calibTextFile, oiTarget, oiLog, &oiDiam)

       DESCRIPTION
       Load the diameters of the calibration stars specified in the ASCII
       file 'calibTextFile'. These calibration stars are compared with
       the stars included in oiTarget.

       PARAMETERS
       - calibTextFile: ASCII text file created by amdlibQueryAllStarDiameters
       - oiTarget     : valid oi_structure loaded with oiFitsLoadFiles.
       - oiLog        : valid oi_structure loaded with oiFitsLoadFiles.
       - oiDiam       : valid oi_structure loaded with oiFitsLoadFiles.

       SEE ALSO
    */
{
    yocoLogTrace,"_amdlibLoadOiDiamFromFile()";
    local id, isCal, calTitles, calNames, calDiamz, calDiamzErr;

    /* Check simples parameters */
    if ( !amdlibCheckCalibTextFile(calibTextFile) )
    {
        yocoError, "Check function's parameters";
        return 0;
    }

    /* Check it */
    if( !open(calibTextFile,"r",1) )
    {
        yocoError,"Cannot find 'calibTextFile'";
        return 0;
    }

    /* Verbose outputs */
    yocoLogInfo,"Reading calibrator diameter from file",calibTextFile;
  
    /* Load the data */
    amdlibReadAsciiFile, calibTextFile, vals, calTitles;
    calNames    = vals(where(calTitles=="Star_name")(1),);
    calDiamz    = yocoStr2Double(vals(where(calTitles=="diam(mas)")(1),));
    calDiamzErr = yocoStr2Double(vals(where(calTitles=="diamErr(mas)")(1),));
  

    /* Find the target flaged as calib in the AMBER files */
    isCal = ( oiFitsGetOiLog(oiTarget, oiLog).dprCatg == "CALIB" );

    /* Fill the structure */
    oiDiam          = array(struct_oiDiam, numberof(oiTarget) );
    oiDiam.targetId = oiTarget.targetId;
    oiDiam.isCal    = 0;
  
    for ( i=1 ; i<=numberof(oiTarget) ; i++)
    {
        /* Associate with oiTarget */
        id = yocoListId( strtrim(oiTarget(i).target), strtrim(calNames) );
    
        if ( !isCal(i) && id==0 ) {
            str = swrite(format="%s (SCIENCE) has no diameter specified.",
                         oiTarget(i).target);
            yocoLogInfo,str,"Will be used as SCIENCE.";
            continue;
        }
        if ( isCal(i) && id==0 ) {
            str = swrite(format="%s (CALIB) has no diameter specified",
                         oiTarget(i).target);
            yocoLogWarning,str,"Will be used as SCIENCE.";
            continue;
        }
        else if ( isCal(i) && id>0 ) {
            str = swrite(format="%s (CALIB) has diameter of %.2f+-%.2fmas.",
                         oiTarget(i).target, calDiamz(id), calDiamzErr(id));
            yocoLogInfo,str,"Will be used as CALIB.";
        }
        else if ( !isCal(i) && id>0 ) {
            str = swrite(format="%s (SCIENCE) has diameter of %.2f+-%.2fmas",
                         oiTarget(i).target, calDiamz(id), calDiamzErr(id));
            yocoLogInfo,str,"Will be used as SCIENCE.";
            continue;
        }
    
        /* fill the diameter */
        oiDiam(i).isCal   = 1;
        oiDiam(i).diam    = calDiamz(id);
        oiDiam(i).diamErr = calDiamzErr(id);
    }

    /* Check if some calibrator are filled */
    if ( allof(oiDiam.isCal==0) ) {
        yocoError,"No calibrators have been found.\n "+
            "Please fill the following file with calibrators (using amdlibSearchAllStarDiameters)",
            calibTextFile;
        return 0;
    }

    yocoLogTrace,"_amdlibLoadOiDiamFromFile done";
    return 1;
}

/* ****************************************************************** */

func amdlibCheckAllWavelengthTable(void, inputDir=, outputDir=, overwrite=, interactive=)
/* DOCUMENT amdlibCheckAllWavelengthTable(void, inputDir=, outputDir=, interactive=)

   DESCRIPTION
   Execute "amdlibCheckWavelengthTable" in loop over the files of the
   directory inputDir. It determines the spectral offset of the
   current wavelength table by comparing the AMBER_SPECTRUM with a
   telluric spectrum.

   Actually, for each spectral setup the function will try to fit all the files
   with this setup (P2VM ID). Then it take the offset of the best fit, and
   apply this offset to all the files with same P2VM_ID.

   This is to ensure that each files with the same spectral setups
   (P2VM_ID) have the same corrected spectral table, which is mandatory
   for a proper calibration of the data afterward.

   An interactive version is available with the option interactive=1
   In this case, the function display the result of the fit and the user
   is asked if he wants to aply the offset to all files (with same P2VM ID).

   PARAMETERS
   - inputDir    : directory with OIDATA_AVG or OIDATA_APP_AVG data
   - outputDir   : output directory with the _SPEC product files.
   - interactive : interactive version if 1

   EXAMPLES
   > amdlibCheckAllWavelengthTable, inputDir="/data/Datas/Datas/2010-01_Rotators/2010-01_Rotators_3.0b5_MedRes_OIDATA_APP_AVG";
   > amdlibCheckAllWavelengthTable, inputDir="/Volumes/Datas/Datas/deltaScoData/deltaScoData_SVN_20110819_11161_OIDATA_APP_AVG", interactive=0;

   FIXME: Deal with Low Resolution. Deal with already
   existing product files (load them in case overwrite=0 ?)

   SEE ALSO amdlib
 */
{
    yocoLogInfo,"amdlibCheckAllWavelengthTable()";

    /* Check simples parameters */
    if ( !amdlibCheckVoidParam(void)     ||
         !amdlibCheckOverwrite(overwrite) ||
         !amdlibCheckGraphicsDpi(graphicsDpi)  )
    {
        yocoError, "Check function's parameters";
        return 0;
    }

    /* Default is non-interactive mode */
    if (is_void(interactive)) interactive=0;
  
    /* Check the inputDir parameter */
    message = "Choose the directory you want to check";
    if ( !amdlibFileChooser(message, inputDir, 3) ||
         !amdlibCheckDir(inputDir, 0) )
    {
        yocoError,"'inputDir' not specified correctly.";
        return 0;
    }
  
    /* Verbose output */
    yocoLogInfo, "Computing OIDATA_AVG_SPEC for directory"+
        " (overwrite="+pr1(overwrite)+") :", inputDir;
  
    /* Check outputDir */
    if ( !amdlibCheckProductDir(outputDir, 1, overwrite, suffix="SPEC", up=1, inputDir=inputDir) )
    {
        yocoError, "Could not construct 'outputDir'";
        return 0;
    }
  
    /* Verbose output */
    yocoLogInfo, "Product directory is:", outputDir;
  
    /* If existing but overwrite is void, ask the user */
    if (is_void(overwrite))
    {
        overwrite = yocoGuiInfoBox("Directory " +
                                   yocoStrrtok(outputDir,"/")(2) + 
                                   " already exists. Re-check all OIDATA_AVGs ?\n",
                                   butText=["YES","NO!"],butReturn=[1,0],
                                   dpi=graphicsDpi);
    }
  
    /* Read the log of the inputDir */
    if ( !amdlibReadLogFromDir(inputDir, data, tit) )
    {
        yocoError, "Could not read logfile of 'inputDir'";
        return 0;
    }
  
    /* Keep only SCIENCE_AVERAGED and CALIB_AVERAGED data */
    proCatg = data( where(tit=="pro_catg")(1), );
    id      = where(strmatch(proCatg,"_AVERAGED") );
    data    = data(,id);
  
    /* Check if any */
    if ( numberof(id)<1 )
    {
        yocoError,"Cannot find any OIDATA_AVG in 'inputDir'.";
        return 0;
    }
  
    /* Build the full path+filenames */
    inputOiFile = data( where(tit=="fileName")(1),  );
    inputOiFile = inputDir + inputOiFile;
    yocoFileSplitName, inputOiFile, inputDir, inputFile, inputSuffix;
    outputOiFile = outputDir + inputFile + inputSuffix;
  
    /* Define the spectral setup with P2VMID */
    setups   = amdlibLogGet( data, tit, "P2VM_ID");
    setupsId = yocoListUniqueId(setups);
  
    /* Define the buttons for the graphical interface */
    bfiles = array( yocoBROWSER_BUTTONS(height=14,font="helvetica",y=0.725,height=14,
                                        dy=0.011,dx=0.07),5);
    bfiles.x = span(0.15,0.95,6)([1,2,4,5,6]);
    bfiles.text = ["Abort","Skip this setup", "Apply to all files", "see next file", "see previous file"];
    bfiles.action = ["q","s","a","+","-"];
  
    /* Prepare the window */
    winkill,1;
    yocoNmCreate,1,1,1,dpi=graphicsDpi,landscape=1,
        width=int(825.0*graphicsDpi/75), height=int(638.0*graphicsDpi/75);
  
    /* Loop on spectral setups */
    for (s=1; s<=max(setupsId); s++)
    {
      
        /* Look for the files with this setup */
        inputOiFileInSetup  = inputOiFile(where(setupsId==s));
        outputOiFileInSetup = outputOiFile(where(setupsId==s));
        nFilesInSetup       = numberof(inputOiFileInSetup);
        dataInSetup         = data(,where(setupsId==s));
      
        /* Loop on files in setup */
        f=1;
        writeTable = 0;
        forceDelta = [];
        outputs = array(0.0, 3, nFilesInSetup);
        while ( f<=nFilesInSetup )
        {
            /* Execute the selection process on this file, with graphical output
               but without producing the product file. The result (delta, snr, pvw)
               is stored in the array outputs. */
            if (amdlibCheckWavelengthTable(output, inputOiFile  = inputOiFileInSetup(f),
                                           outputOiFile  = "",
                                           forceDelta = forceDelta,
                                           checkPlot=1) == 0)
            {
                yocoError, "Could not check the wavelength table for file:",inputOiFileInSetup(f);
                f++;
                continue;
            }
            else
            {
                outputs(,f) = output;
            }
          
            /* If non-interactive, simply go to next file */
            if ( interactive==0 ) {f++; continue; }
          
            /* If interactive, plot additional information and buttons,
               directly in the plot built by amdlibCheckWavelengthTable. */            
            str = "Spectral setup: "+pr1(s)+"/"+pr1(max(setupsId));
            plt,str,0.2,0.78,justify="CH",height=16,tosys=0,font="helveticaB";

            str = "File "+pr1(f)+"/"+pr1(nFilesInSetup)+" in this setup";
            plt,str,0.8,0.78,justify="CH",height=16,tosys=0,font="helveticaB";

            button_plot, bfiles;
            redraw;
          
            /* Call mouse, get answer and process it */
            answer = amdlibButtonAnswer(bfiles);
            if ( answer=="+" ) {
                f++;
                if (f>nFilesInSetup) { writeTable = 0; break;}
            }
            else if ( answer=="-" ) {
                f--;
                if (f==0) {s=max(0,s-2); writeTable = 0; break;}
            }
            else if ( answer=="q" ) {
                winkill,1;
                return 0;
            }
            else if ( answer=="a" ) {
                writeTable = 1;
                break;
            }
            else if ( answer=="s" ) {
                writeTable = 0;
                break;
            }
          
            /* FIXME: add possibility to update manually the offset
               by updating forceDelta */
        }
        /* End loop on files for this setup */

      
        /* If automatic, will look for the best fit (best SNR)
           and will apply its correction to all files */
        if ( interactive==0 )
        {
            id    = outputs(2,)(mxx); 
            delta = outputs(1,id);
            snr   = outputs(2,id);
          
            if (snr>2.0)
            {
                writeTable = 1;
                yocoLogInfo,"Found best fit with teluric spectra in file (snr="+pr1(snr)+"):",
                    inputOiFileInSetup(id);
            }
            else
            {
                writeTable = 0;
                yocoLogWarning,"No correct fit with teluric spectra in any of the files.";
            }
        }
      
        /* Eventually continue directly to next setup, this means the OIDATA_AVG_SPEC
           files for the current setup will not be created. */
        if (writeTable==0)
        {
            yocoLogWarning,"Don't update OI_WAVELENGTH table for this setup";
            continue;
        }
      
        /* Else loop on files to update the wavelength tables.
           This is done with the amdlibCheckWavelengthTable function
           so that a PDF is also produced. */
        for (f=1; f<=nFilesInSetup; f++)
        {
            if (amdlibCheckWavelengthTable(inputOiFile = inputOiFileInSetup(f),
                                           outputOiFile = outputOiFileInSetup(f),
                                           forceDelta = delta,
                                           checkPlot=1) == 0)
            {
                yocoError, "Could not update the wavelength table for file",inputOiFileInSetup(f);
                continue;
            }
        }
      
    }
    /* End loop on setups */
  
    /* End */
    yocoLogInfo,"amdlibCheckAllWavelengthTable done";
    winkill,1;
    return 1;
}

/* **************************************************************** */

func amdlibCheckWavelengthTable(&output, inputOiFile=, outputOiFile=, checkPlot=, forceDelta=, catalogsDir=)
/* DOCUMENT amdlibCheckWavelengthTable(&output, inputOiFile=, outputOiFile=, checkPlot=, catalogsDir=)

   DESCRIPTION
   Compare the AMBER_SPECTRUM in OIDATA_AVG files with a telluric spectra
   an update the wavelength table with an optimal offset (offset only, no
   fit of the dispersion).

   The product is an OIDATA_AVG_SPEC file with an updated OI_WAVELENGTH
   table, stored in outputOiFile.

   Note that if you execute this function independently on the SCI and
   the CAL files of an observation, the product files may have slightly
   *different* wavelength table and the calibration will be impossible.

   Use amdlibCheckAllWavelengthTable instead (or use the parameter
   forceDelta= on both files to force the spectral offset to a given value).

   PARAMETERS
   - inputOiFile  : input OIDATA_AVG file
   - outputOiFile : output OIDATA_AVG_SPEC file 
   - forceDelta   : to use a given spectral offset (mum).

   FIXME: Deal with Low Resolution. Deal with already
   existing product files (load them in case overwrite=0 ?)

   EXAMPLES

   SEE ALSO amdlib, amdlibCheckAllWavelengthTable
 */
{
    yocoLogInfo,"amdlibCheckWavelengthTable()";
    local inputDir, outputDir, suffix, compress;
    local catalogsDir, graphicsDpi;
    compress = 0;
    output   = [];

    /* Check simples parameters */
    if ( !amdlibCheckVoidParam(void)  ||
         !amdlibCheckCatalogsDir(catalogsDir) || 
         !amdlibCheckGraphicsDpi(graphicsDpi)  )
    {
        yocoError, "Check function's parameters";
        return 0;
    }
    
    /* Check the obs file(s) */
    message = "OIDATA_AVG FILE\n Enter an observation file";
    if (amdlibFileChooser(message, inputOiFile, 2) == 0)
    {
        yocoError,"'inputOiFile' not specified correctly.";
        return 0;
    }

    /* Create outputDir if not given by the user
       all product will be in the same directory given by the
       name of the first inputOiFile */
    if( is_void(outputOiFile) )
    {
        yocoFileSplitName, inputOiFile, inputDir, outputFile, suffix;
      
        /* check outputDir */
        if ( !amdlibCheckProductDir(outputDir, 1, suffix="SPEC", up=1, inputDir=inputDir(1)) )
        {
            yocoError, "Could not construct the directory for the OIDATA_AVG product";
            return 0;
        }
      
        outputOiFile = outputDir + outputFile + suffix;
    }
    else if (outputOiFile != "")
    {
        yocoFileSplitName, outputOiFile, outputDir, outputFile, suffix;
        outputOiFile = outputDir + outputFile + suffix;
    }
  
    /* Open file in append mode so that we can correct its table */
    yocoLogInfo,"Load file:",inputOiFile;
    fh = cfitsio_open(inputOiFile,"r");

    /* Read the spectral resolution to decide how to convolve
       the reference spectra. Reject Low_resolution data. */
    cfitsio_goto_hdu,fh,1;
    insMode = cfitsio_get(fh,"HIERARCH ESO INS MODE");
    if (strmatch(insMode,"Medium"))
    {
        R = 1200;
        alpha = 1.0025;
    }
    else if (strmatch(insMode,"High"))
    {
        R = 12000;
        alpha = 0.992;
    }
    else if (strmatch(insMode,"Low"))
    {
        R = 35.0;
        alpha = 1.1;
        yocoError,"Cannot deal with Low Resolution data.";
        return 0;
    }
    else
    {
        yocoError,"insMode "+pr1(insMode)+" is not known.";
        return 0;
    }
  
    /* Read AMBER spectrum with its wavelength table */
    cfitsio_goto_hdu,fh,"AMBER_SPECTRUM";
    bin = cfitsio_read_bintable(fh);
    cfitsio_close,fh;
    spt = (*bin(3))(avg,);
    lbd = (*bin(1)) * 1e6;

    /* Keep only 1.3 to 2.5mum part of the spectrum,
       and divide by median. */
    spt = spt(where(lbd>1.3 & lbd<2.5));
    lbd = lbd(where(lbd>1.3 & lbd<2.5));
    spt /= median(spt);

    /* Compress slightly lbd */
    lbd = alpha*(lbd-lbd(avg)) + lbd(avg);
    lbd0 = lbd(avg);
  
    /* High Pass Filter AMBER spectra. This is to ensure we
       are comparing lines and not continuum. We also force
       the 10 pixels at the edges of the spectra at 0.
       This filtering is not performed in LowRes. */
    nlbd = numberof(lbd);
    freq = (u= indgen(0:nlbd-1)) - nlbd*(u > nlbd/2);
    if ( strmatch(insMode,"Low") )
        sptn = spt - spt(avg,-,);
    else 
        sptn = fft(fft(spt) * (abs(freq)>20) , -1).re * (abs(freq)>10) / nlbd;

    /* FIXME: Filter the fringes for observations that are affected
       by wiggles... can be checked from dates */

    /* Use sigma-clipping so that obvious outliers (>7.rms, that is bad pixels)
       are removed from the fit. The outliers are replaced by an interpolation. */
    id = abs(sptn)>7*sptn(rms);
    if ( is_array( where(id) ) ) {
        sptn( where(id) ) = interp( sptn(where(!id)), lbd(where(!id)), lbd(where(id)) );
    }

    /* Read the TELURRIC spectra for PVW of 1..8mm
       Its wavelength is already in mum. */
    fhRef = cfitsio_open(catalogsDir+"/telluric_models_JHK.fits[TELLURIC_SPECTRA]");
    Rref  = 40000.0;
    bin = cfitsio_read_bintable(fhRef);
    cfitsio_close, fhRef;
    lbdRef0 = *bin(1);
    sptRef0 = [*bin(2),*bin(3),*bin(4),*bin(5),*bin(6),*bin(6),*bin(8),*bin(9)];

    /* Degrade spectral resolution of the TELURRIC spectra to 
       match the AMBER one. Convolution by a Gaussian with
       fwhm in pixel given by the ration of resolution  */
    sptRef0 = gaussm(sptRef0,[Rref/R],fwhm=1);

    /* Project TELURRIC SPECTRA into the wavelength table of AMBER */
    sptRef  = interp(sptRef0, lbdRef0, lbd, 1);
    sptRef /= median(sptRef,1)(-,);

    /* High Pass Filter TELURRIC spectra, with the same filter
       as AMBER spectra. Also cut the edges. This filtering
       is not performed in LowRes. */
    if ( strmatch(insMode,"Low") )
        sptRefn = sptRef - sptRef(avg,-,);
    else
        sptRefn = fft(fft(sptRef,[1,0]) * (abs(freq)>20) , [-1,0]).re * (abs(freq)>10) / nlbd;

    /* Convolution between the HighPass filtered AMBER
       and REFERENCE spectra (for 1..8mm of PWV). */
    xc   = ( indgen(numberof(lbd)) - numberof(lbd)/2-1 ) * median(lbd(dif));
    conv = convoln(sptRefn, sptn(::-1,-) );
    conv /= conv(rms,-,);
  
    /* Get the best fit of pwv (best convolution). Find the position
       of the maximum of convolution function for this pwv. Define
       a snr as the ratio between maximum value and rms  */
    pwv   = conv(max,)(mxx);
    Mxx   = conv(,pwv)(mxx);
    snr   = conv(,pwv)(max) / conv(,pwv)(rms);
    delta = xc( Mxx );

    /* **** START Experimental ****
       Additional non-linear fit to better recover the offset
       and correct the linear dispersion. Main issue is that
       lmfit is missing in yorick distribution */
    // A = [delta, 1.0];
    // result= lmfit(__get, , A, sptn, fit=[1]);
    // write,pr1(delta*1e3)+"    "+pr1(A(1)*1e3)+"  "+pr1(A(2));
    // delta = A(1);
    /* **** END Experimental **** */
  

    /* Eventually force delta if provided */
    if ( is_array(forceDelta) ) delta=forceDelta;

    /* Build the output array */
    output = [delta, snr, pwv];

    /* Plot results */
    if (checkPlot)
    {
        yocoNmCreate,checkPlot,1,3,landscape=1,dy=[0.04,0.01];
        fma;

        /* Plot the convolution function */
        plsys,1;
        plg, conv(,pwv), xc;
        plg, conv(,1), xc, type=3;
        plg, conv(,0), xc, type=3;
        yocoPlotVertLine,delta,color="green",width=2;
        xytitles,,"Correlation Function",[0.01,0.015];
        limits;
    
        /* Plot the match between AMBER and TELURRIC spectra */
        plsys,2;
        plg,sptRef(,pwv),lbd,color="red";
        plg,spt,lbd+delta;
        limits; range,0.8,1.2;
        xytitles,,"Spectra",[0.01,0.015];
    
        /* Plot the match between the filtered version of the AMBER and
           TELURRIC spectra (on which the convolution is actually done) */
        plsys,3;
        plg,sptRefn(,pwv),lbd,color="red";
        plg,sptn,lbd+delta;
        limits;
        xytitles,"!l (!mm)","Spectra (filtered)",[0.01,0.015];

        /* Some additional information */
        plt,"!l_0_="+pr1(lbd0)+"!mm   !d="+pr1(delta*1e3)+"nm  snr="+pr1(snr),
            0.8,0.755,tosys=0,height=14,font="helvetica",justify="CH";
        plt,yocoStrReplace(insMode,"_","!_"),
            0.2,0.755,tosys=0,height=14,font="helvetica",justify="CH";
    }

    /* Create outputFile. The case outputOiFile="" is for the call
       within amdlibCheckAllWavelengthTable. */
    if ( outputOiFile != "" )
    {
        yocoLogInfo,"Create the OIDATA_AVG_SPEC file:",outputOiFile;
      
        /* Copy inputOiFile into outputOiFile */
        system,"rm -rf " + outputOiFile;
        system,"cp -rf " + inputOiFile + " " + outputOiFile;
        system,"chmod u+w " + outputOiFile;

        /* Handle compressed file */
        if ( !amdlibUncompress(outputOiFile,compress) )
        {
            yocoError,"OIDATA_AVG file 'outputOiFile' cannot be uncompressed for dumping new wavelength table.";
            return 0;
        }
      
        /* Update wavelength table in OI_WAVELENGTH */
        fh = cfitsio_open(outputOiFile,"a");
        cfitsio_goto_hdu,fh,"OI_WAVELENGTH";
        lbd = *cfitsio_read_bintable(fh)(1);
        cfitsio_write_col, fh, 1, lbd + delta*1e-6;

        /* Update wavelength table in AMBER_SPECTRUM */
        cfitsio_goto_hdu,fh,"AMBER_SPECTRUM";
        lbd = *cfitsio_read_bintable(fh)(1);
        cfitsio_write_col, fh, 1, lbd + delta*1e-6;
        cfitsio_close,fh;

        /* Handle compressed file */
        if ( !amdlibCompress(outputOiFile,compress) )
        {
            yocoError, "Could not recompress the output OIDATA_AVG file";
            return 0;
        }
      
        /* Store the plot */
        if (checkPlot) {
            here = get_cwd();
            cd,outputDir;
            pdf,outputFile+".pdf";
            cd,here;
        }
    }
  
    yocoLogTrace,"amdlibCheckWavelengthTable done";
    return 1;
}

/* **************************************************************** */

func amdlibButtonAnswer(but,click)
{
    if (is_void(click)) click = mouse(,,"click a button");
  
    /* Loop on the buttons */
    for ( i=1 ; i<=numberof(but) ; i++ )
    {
        if ( abs(click(6) - but(i).y) < but(i).dy &&
             abs(click(5) - but(i).x) < but(i).dx )
            return but(i).action;
    }
    /* default return "" */
    return "";
}

